/*
******************************************************
file:   bdd.h

header for BDDs
******************************************************
*/



#ifndef bdd_h
#define bdd_h



#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <time.h>

#define boolean int


typedef void *bdd;

#ifndef TYPE_BDD_CONTEXT
#define TYPE_BDD_CONTEXT
typedef bdd (*bdd_context)(bdd *);
#endif


/*   Boolean operations  */
extern
bdd AND(bdd G1, bdd G2);

extern
bdd EQ(bdd G1, bdd G2);

extern
bdd IMP(bdd G1, bdd G2);

extern
bdd NAND(bdd G1, bdd G2);

extern
bdd NOR(bdd G1, bdd G2);

extern
bdd NOT(bdd G);

extern
bdd OR(bdd G1, bdd G2);

extern
bdd XOR(bdd G1, bdd G2);

extern
bdd duplicate(bdd G);


extern
bdd if_then_else(bdd G0, bdd G1, bdd G2);


extern
bdd ifter(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
);


/*   Booleans  */
extern
bdd one();

extern
bdd varx(int i);

extern
bdd zero();


extern
int is_var(bdd G, unsigned int *k);



/*  delete bdd  */
extern
void delete_bdd(bdd G);


/*   bdd info   */
extern
void get_bdd_info(bdd G, int *info);

extern
boolean is_bdd_one(bdd G); 

extern
boolean is_bdd_zero(bdd G);

extern
int bdd_equal(bdd G0, bdd G1); 

extern
void print_bdd(bdd G);

extern int size(bdd G);

extern int size_of_vertex();

extern void printinfo(bdd G);

extern void printCPUtime();

extern void checkCPUtime();

extern void print_bdd_info();

extern
int bdd_root_var_index(bdd G);





/*  start bdds environment  */
extern
void start_environment();

extern
void start_bdd_env(
unsigned int *optval,
double *optfval
);


/*  bindings and fixed points  */

extern
bdd for_all(int i, bdd F);

extern
bdd compose(bdd F, int i, bdd G);

extern
bdd compose_vct(bdd F, int *x, bdd *G, int n);

extern
bdd compose_var_vct(bdd F, int *x, int *y, int n);

extern
bdd there_exists(int i, bdd F);

extern
bdd there_exists_vct(int *x, int n, bdd F);


extern
void least_fixed_point(bdd_context F, bdd *G, int i, int *steps);

extern
void least_fixed_point_step(bdd_context F, bdd *G, int i, int *steps);


extern
void lfp_step_step(
bdd_context F, 
bdd *G, 
int i, 
int max_steps, 
int *steps
);

extern
bdd restrict(bdd F, int i, int rbit);

extern
bdd shift_variables(bdd F, int k);





/* 
Evaluate bdd f 
with boolean values in x.
Size x =  n
*/


int eval_bdd(bdd f, int *x, int n);

void eval_bdd_vct(int k, int *y, bdd *f, int *x, int n);








/* Procedures for multiple bdd printing   */

void open_mprbdd();
void close_mprbdd();
void mfprint_bdd(FILE *pfile, bdd G);




extern
void satisfy_all(
FILE *satfile,
bdd G 
);


void ord_satisfy_all(
FILE *satfile,
bdd G,
unsigned int maxarg,  /* max number of bdd variables expected */
unsigned int size_ord,  /* size of ord_array */
unsigned int *ord_array
);



void bdd2c(
	FILE *pfile, 
	bdd G, 
	unsigned int maxarg,
	unsigned int size_ord, 
	unsigned int *ord_array,
        char *fname
);




/*  
***************************************************
To be implemented:


extern int Satisfy_count(bdd G, int n);

extern int Satisfy_one(bdd G, void *x);


etc........
***************************************************
*/




/*
**************************************
Functions for multiple variable systems
***************************************
*/



extern
bdd compose_x(bdd F, int i, bdd G);

extern
bdd compose_y(bdd F, int i, bdd G);

/*
*********************************
Shift indexes of ALL variables.
Note it is not possible to shift
indexes only for one system
of variables.
*********************************
*/

extern
bdd shift_vars(bdd F, int k);


extern
bdd exists_x(int i, bdd F);


extern
bdd exists_y(int i, bdd F);


extern
bdd forall_x(int i, bdd F);

extern
bdd forall_y(int i, bdd F);

extern
bdd restrict_x(bdd F, int i, int rbit);

extern
bdd restrict_y(bdd F, int i, int rbit);


extern
bdd bddvar_x(int i);

extern
bdd bddvar_y(int i);


extern
bdd ifter_x(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
);


extern
bdd ifter_y(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
);





extern
int is_var_x(bdd G, unsigned int *k);

extern
int is_var_y(bdd G, unsigned int *k);





extern
int get_num_bdd_vtx();









/*
********************************************************
File  compatibility  is included to assure compatibily 
between new and old names for primitives on BDDs.
********************************************************
*/



#include "compatib.h"


#endif
