
/*
**
**  file look.up.table.h
**
**
**  primitives for look_up_table
**
*/


#ifndef bmath_h
#define bmath_h


#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "bdd.h"




void conv_bin_to_bdds(int num_bits, int *a, bdd *b);

void conv_dec_to_bin(int num_bits, unsigned int a_in, int *b_out);

unsigned int conv_bdds_to_dec(int num_bits, bdd *b);

void conv_dec_to_bin(int num_bits, unsigned int a_in, int *b_out);


unsigned int conv_bin_to_dec(int num_bits, int *b);



/*
******************************************
Gets a function vector from a relation.

Relation represented by bdd G.
Result in F s.t.
replacing for i = 0, ... n - 1
variable var_idx[i] with F[i] 
in G
satisfies G.


******************************************
*/


void determinize_bdd(
int n,   /*  size var_idx, Y   */
int arity,  /*   arity of G  */
int *var_idx,    /* indexes of variables to be determinized  */
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
);


void det_plant(
int n,   
int r,   
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
);


void det_controller(
int n,   
int r,   
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
);


void test_bin_op(
int size_in,
int size_out, 
bdd *F
);


void sim_closed_loop(
int k,   /*   number of state components   */
int r,   /* size event vector  */
int n,   /* size state vector (for each component)  */
bdd *P,   /* plant transition function  */
bdd *F   /* controller  */
);

void print_state(int k, int n, int *x);

void print_event(int r, int *u);

void test_leq(
int size_in,
bdd *F
);


void sim_open_loop(
int k,   /*   number of state components   */
int r,   /* size event vector  */
int n,   /* size state vector (for each component)  */
bdd *P,  /* plant transition function  */
int max_test   /* number of test carried out  */
);

void display_bf(
int n,   /*  arity boolean function   F   */
bdd F   /* relation    */
);


void display_plant(
int k,   /* state components  */
int r,  /*  size event vector  */
int n,   /*  size state vector for each component   */
bdd P   /* plant transition relation    */
);


void rand_display_plant(
int k,   /* state components  */
int r,  /*  size event vector  */
int n,   /*  size state vector for each component   */
bdd P,   /* plant transition relation    */
int max_test   /* number of tests to be carried out  */
);


void display_1_bf(
int k,   /* arguments   */
int n,   /*  size of each argument   */
bdd F   /* boolean function of arity k*n   */
);


#endif








