/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 0

/*
******************************************
Gets a deterministic controller from
a nondeterministic one.

r = size event vector (u);
n = size state vector (x).

variable order:
u_0, ... u_(r - 1), x_0, x'_0, ... x_(n - 1), x'_(n - 1)

******************************************
*/


void det_controller(
int n,   
int r,   
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
)

{  /*  begin */

/* declarations  */


int i;
int *var_idx;

/*  program  */

#if (dbg_flag > 100)
printf("\ndet_controller: ckp 100 (begin)\n");
#endif


/* define variables  */

var_idx = (int *) malloc(r * sizeof(int));

if (var_idx == NULL)
{
    printf("\n\nError: no space for var_idx (det_controller)");
    exit (1234);
}

/* var_idx != NULL  */

for (i = 0; i < r; i++)
	{
	   var_idx[i] = i;
	}
	
/* define F   */

determinize_bdd(r, r + n, var_idx, F, G);

free(var_idx);

}  /*  end  */




