/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 0
#define dbg_plant 0


/*
******************************************
Gets a deterministic plant from
a nondeterministic one.

r = size event vector (u);
n = size state vector (x).

variable order:
u_0, ... u_(r - 1), x_0, x'_0, ... x_(n - 1), x'_(n - 1)

******************************************
*/


void det_plant(
int n,   
int r,   
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
)

{  /*  begin */

/* declarations  */


int i;
int *var_idx;

/*  program  */

#if (dbg_flag > 100)
printf("\ndet_controller: ckp 100 (begin)\n");
#endif


#if (dbg_plant > 10)
var_idx = (int *) malloc((r + 2*n) * sizeof(int));

if (var_idx == NULL)
{
    printf("\n\nError: no space for var_idx (det_controller)");
    exit (1234);
}

/* var_idx != NULL  */

for (i = 0; i < r + 2*n; i++)
	{
	   var_idx[i] = 0;
	}
	
printf("\n\ndet_plant: plant value = %d",
eval_bdd(G, var_idx, r + 2*n)
);

free(var_idx);
#endif








/* define variables  */

var_idx = (int *) malloc(n * sizeof(int));

if (var_idx == NULL)
{
    printf("\n\nError: no space for var_idx (det_controller)");
    exit (1234);
}

/* var_idx != NULL  */

for (i = 0; i < n; i++)
	{
	   var_idx[i] = r + 1 + 2*i;
	}
	
/* define F   */

determinize_bdd(n, r + 2*n, var_idx, F, G);

free(var_idx);

}  /*  end  */




