/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 60

/*
******************************************
Gets a function vector from a relation.

Relation represented by bdd G.
Result in F s.t.
replacing for i = 0, ... n - 1
variable var_idx[i] with F[i] 
in G
satisfies G.


HYP:
var_idx[0] < var_idx[1] < ...  var_idx[n - 1]
******************************************
*/


void determinize_bdd(
int n,   /*  size var_idx, F   */
int arity,  /*   arity of G  */
int *var_idx,    /* indexes of variables to be determinized  */
bdd *F,   /* result of determinization  */
bdd G   /* relation    */
)

{  /*  begin */

/* declarations  */


bdd buffer[5];
int h, i, j, ii;


/*  program  */

#if (dbg_flag > 100)
printf("\ndeterminize_bdd: ckp 100 (begin)\n");
#endif

/* define F   */

buffer[1] = one();

for (j = 0; j < n; j++)
	{

/* define F[j]  */

#if (dbg_flag > 1000)
printf("\ndeterminize_bdd: ckp 500.%d\n", j);
#endif

buffer[2] = duplicate(G);  

	for (i = 0; i < j; i++)
		{
    	 buffer[3] = compose(buffer[2], var_idx[i], F[i]);
    	 delete_bdd(buffer[2]);
    	 buffer[2] = buffer[3];
		}

#if (dbg_flag > 1000)
printf("\ndeterminize_bdd: ckp 600.%d\n", j);
#endif


buffer[3] = compose(buffer[2], var_idx[j], buffer[1]);
      delete_bdd(buffer[2]);
      buffer[2] = buffer[3];

	for (i = j + 1; i < n; i++)
		{
    	 buffer[3] = there_exists(var_idx[i], buffer[2]);
    	  delete_bdd(buffer[2]);
    	  buffer[2] = buffer[3];
		}

#if (dbg_flag > 1000)
printf("\ndeterminize_bdd: ckp 700.%d\n", j);
#endif

F[j] = buffer[2];
	}


  delete_bdd(buffer[1]);


#if (dbg_flag > 100)
printf("\ndeterminize_bdd: ckp 5000\n");
#endif


/*  recompact variables   */
for (j = 0; j < n; j++)
{   /*  for j  */

h = 0;  /*  slot to be occupied  */
ii = 0;   /* next var_idx   */

	for (i = 0; i < arity; i++)
	{   /* for i  */
		
		if ( i == var_idx[ii] )
		{
		    ii++;
		    continue;
		}

		/* i is not a variable in var_idx  */
		
#if (dbg_flag > 50)
printf("\ndeterminize_bdd: ckp 100: F[%d]: %d <-- %d\n", j, i, h);
#endif
		if (h < i)
		{  /*  shift  */

		 buffer[0] = varx(h);
		 buffer[1] = compose(F[j], i, buffer[0]);
		 delete_bdd(buffer[0]);
		 delete_bdd(F[j]);
		 F[j] = buffer[1];
		 h++;
		 continue;
		}
	
	/* h == i  */
	       h++;
	       
	}   /* for i  */
	
}   /*  for j  */



#if (dbg_flag > 100)
printf("\ndeterminize_bdd: ckp 50000 (end)\n");
#endif

}  /*  end  */




