/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 0


/*
******************************************
Display plant transition relation.


u : event vector (size r)
x : state vector  (size k*n)

Variable order:
u_0, ... u_{n - 1}
x_{0, 0}, ... x_{0, n - 1}, ... 
x_{k - 1, 0}, ... x_{k - 1, n - 1}


HYP: r + 2*n <= 32
******************************************
*/


void rand_display_plant(
int k,   /* state components  */
int r,  /*  size event vector  */
int n,   /*  size state vector for each component   */
bdd P,   /* plant transition relation    */
int max_test   /* number of tests to be carried out  */
)

{  /*  begin */

/* declarations  */


unsigned int i, max_in, hh;
int *ux, *x, *u, *z;
int result, j;

/*  program  */

#if (dbg_flag > 100)
printf("\ndisplay_bf: ckp 100 (begin)\n");
#endif

x = (int *) malloc(k*n * sizeof(int));

if (x == NULL)
{
    printf("\n\nError: no space for x (display_bf)");
    exit (1234);
}

u = (int *) malloc(r * sizeof(int));

if (u == NULL)
{
    printf("\n\nError: no space for u (display_bf)");
    exit (1234);
}

ux = (int *) malloc((r + 2*k*n) * sizeof(int));

if (ux == NULL)
{
    printf("\n\nError: no space for ux (display_bf)");
    exit (1234);
}

/* ux != NULL  */

/* define max_in   */
max_in = 1;
for (i = 0; i < r + 2*k*n; i++)
{
    max_in = 2*max_in;
}

hh = 0;
while (hh < max_test)
{
 	 	/* define ux  */
 	 	for (i = 0; i < r + 2*k*n; i++)
 	 	{
 	 	    ux[i] = rand()%2;
 	 	} 
 		
 		/* define result  */
 		result = eval_bdd(P, ux, r + 2*k*n);
 		
 		if (result   /*  != 0  */)
 		{   /*  print input   */

 	 	printf("\n\n\nfound %u: transition %u ***************",
 	 	hh, conv_bin_to_dec(r + 2*k*n, ux)
 	 	);

 	 	/*  define present state vector  */
 	 	for (j = 0; j < k*n; j++)
 	 	{
 	 	     x[j] = ux[r + 2*j];
 	 	}
 	 	
 	 	/* print present state vector   */
 	 	printf("\npresent state");
 	 	print_state(k, n, x);

 	 	/*  define event vecor  */
 	 	for (j = 0; j < r; j++)
 	 	{
 	 	     u[j] = ux[j];
 	 	}
 	 	
 	 	/* print event vector   */
 	 	print_event(r, u);


 	 	/*  define next state vector  */
 	 	for (j = 0; j < k*n; j++)
 	 	{
 	 	     x[j] = ux[r + 2*j + 1];
 	 	}
 	 	
 	 	/* print next state vector   */
 	 	printf("\nnext state");
 	 	print_state(k, n, x);
		
		/* update counter   */
		hh++;

 		}   /*  end print input   */

}


free(x);
free(u);
free(ux);

}  /*  end  */




