/*
*************************************
file:  actions.c
*************************************
*/



#include "bmath.h"


#define dbg_flag 0

/*
******************************************
Simulator for plant
in open-loop configuration.

u : event vector (size r)
x : state vector  (size k*n)

Variable order:
u_0, ... u_{n - 1}
x_{0, 0}, ... x_{0, n - 1}, ... 
x_{k - 1, 0}, ... x_{k - 1, n - 1}
******************************************
*/

void sim_open_loop(
int k,   /*   number of state components   */
int r,   /* size event vector  */
int n,   /* size state vector (for each component)  */
bdd *P,  /* plant transition function  */
int max_test   /* number of test carried out  */
)

{  /*  begin */

/* declarations  */


int i, j, h, not_end, jj;
int *x, *u, *z, *ux;
int max_in, hh;

/*  program  */

#if (dbg_flag > 100)
printf("\ndet_controller: ckp 100 (begin)\n");
#endif

/* Define input, output vector    */ 

x = (int *) malloc(k*n * sizeof(int));
if (x == NULL)
{ 
    printf("\n\nError:  no space for x (sim_closed_loop)");  
    exit (1234);
}

u = (int *) malloc(r * sizeof(int));
if (u == NULL)
{ 
    printf("\n\nError:  no space for u (sim_closed_loop)");  
    exit (1234);
}

ux = (int *) malloc((r + k*n)* sizeof(int));
if (ux == NULL)
{ 
    printf("\n\nError:  no space for ux (sim_closed_loop)");  
    exit (1234);
}

z = (int *) malloc(k*n * sizeof(int));
if (z == NULL)
{ 
    printf("\n\nError:  no space for z (sim_closed_loop)");  
    exit (1234);
}




for (hh = 0; hh < max_test; hh++)
{
/* define state   */
for (i = 0; i < k*n; i++)
{
    x[i] = rand()%2;
}

	/*  print present state  */
	printf("\n\n\n\ntest %d: present state", hh);
	print_state(k, n, x);
	
	/* define event  */
for (i = 0; i < r; i++)
{
    u[i] = rand()%2;
}

	/* print event   */ 
	print_event(r, u);
	
	
	/* compute next state  */
	for (jj = 0; jj < r; jj++) { ux[jj] = u[jj]; }
	for (jj = 0; jj < k*n; jj++) { ux[r + jj] = x[jj]; }
	eval_bdd_vct(k*n, z, P, ux, r + k*n);

	printf("\ntest %d: next state", hh);
	print_state(k, n, z);
       
}    /*  for hh  */


free(x);
free(u);
free(ux);
free(z);


}  /*  end  */



