/*
*************************************************
file:  booleop.h
*************************************************
*/


#ifndef booleop_h
#define booleop_h



#include <stdio.h>
#include <stdlib.h>
#include <limits.h>



#include "bddhdr.h"

#include "getbdd.h"


#ifndef TYPE_BDD_CONTEXT
#define TYPE_BDD_CONTEXT
typedef bdd (*bdd_context)(bdd *);
#endif


extern
bdd AND(bdd G1, bdd G2);

extern
bdd EQ(bdd G1, bdd G2);

extern
bdd IMP(bdd G1, bdd G2);

extern
bdd NAND(bdd G1, bdd G2);

extern
bdd NOR(bdd G1, bdd G2);

extern
bdd NOT(bdd G);

extern
bdd OR(bdd G1, bdd G2);

extern
bdd XOR(bdd G1, bdd G2);

extern
bdd for_all(int i, bdd F);

extern
bdd compose(bdd F, int i, bdd G);

extern
bdd compose_var_vct(bdd F, int *x, int *y, int n);

extern
bdd compose_vct(bdd F, int *x, bdd *G, int n);


extern
bdd there_exists(int i, bdd F);

extern
bdd there_exists_vct(int *x, int n, bdd F);


extern
void least_fixed_point(bdd_context F, bdd *G, int i, int *steps);


extern
void least_fixed_point_step(bdd_context F, bdd *G, int i, int *steps);


extern
void lfp_step_step(
bdd_context F, 
bdd *G, 
int i, 
int max_steps, 
int *steps
);


extern
bdd restrict(bdd F, int i, int rbit);

extern
bdd shift_variables(bdd F, int k);



/* 
Evaluate bdd f 
with boolean values in x.
Size x =  n
*/


extern
int eval_bdd(bdd f, int *x, int n);


/* 
Evaluate componentwise bdds in f 
with boolean values in x.
Size x =  n
Size f = size y = k
*/


void eval_bdd_vct(int k, int *y, bdd *f, int *x, int n);








/*
**************************************
Functions for multiple variable systems
***************************************
*/



extern
bdd compose_x(bdd F, int i, bdd G);

extern
bdd compose_y(bdd F, int i, bdd G);


/*
*********************************
Shift indexes of ALL variables.
Note it si not possible to shift
indexes only for one system
of variables.
*********************************
*/

extern
bdd shift_vars(bdd F, int k);



extern
bdd exists_x(int i, bdd F);


extern
bdd exists_y(int i, bdd F);


extern
bdd forall_x(int i, bdd F);

extern
bdd forall_y(int i, bdd F);

extern
bdd restrict_x(bdd F, int i, int rbit);

extern
bdd restrict_y(bdd F, int i, int rbit);


extern
bdd bddvar_x(int i);

extern
bdd bddvar_y(int i);


extern
bdd ifter_x(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
);


extern
bdd ifter_y(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
);




extern
int is_var_x(bdd G, unsigned int *k);

extern
int is_var_y(bdd G, unsigned int *k);






#endif
