/*
****************************************
file: lfp
****************************************
*/

/*  
**  input to least_fixed_point is:
**  a function from BDDs to BDDs
**
** i.e.:typedef bdd (*bdd_context)(bdd *).
**
**  Fixed point computed respect to i-th component of G
*/



#include "local.h"



void least_fixed_point(bdd_context F, bdd *G, int i, int *steps)
/*
**  returns bdd
**  representing the least solution X to 
**
**  compose(F, i, X) = X
**
**  HYP:
**  F formally monotone in x(i).
*/
{
/*  declarations  */

/*  program  */

G[i] = zero();

least_fixed_point_step(F, G, i, steps);

}



void least_fixed_point_step(bdd_context F, bdd *G, int i, int *steps)
/*
**  returns bdd
**  representing the least solution X to 
**
**  compose(F, i, X) = X
**
**  HYP:
**  F formally monotone in x(i).
*/

{   /*  begin program  */

/*  declarations  */
bdd new_result;
int test;

/*  program  */

*steps = 0;

new_result = F(G);
test = bdd_equal(G[i], new_result);


while ( !test )
    {
/*  G[i] != new_result  */
 
       *steps = (*steps) + 1;
        
       delete_bdd(G[i]); 
       
       G[i] = new_result;
       
       new_result = F(G);
              
       test = bdd_equal(G[i], new_result);
    }
    
/*  EQ(G[i], new_result)  */

delete_bdd(new_result); 

/*  return result fixed point in G[i]   */


}  /*  end program  */  
    








/*  
***************************************************

Computes least fix point of F respect to varx(i) in at most 
max_steps steps. 
If fix point reached 
then   steps_done gives the number of steps
       nedded to reach the fixpoint and the fix point is returned;
else   steps_done is not defined and NULL is returned;

***************************************************
*/


void lfp_step_step(
bdd_context F, 
bdd *G, 
int i, 
int max_steps, 
int *steps
)

{   /*  begin program  */

/*  declarations  */
bdd new_result; 
int test;


/*  program  */

*steps = 0;

new_result = F(G);
test = bdd_equal(G[i], new_result);


while ( (!test) && (*steps < max_steps))
    {
/*  G[i] != new_result  */
 
       *steps = (*steps) + 1;
        
       delete_bdd(G[i]); 
       
       G[i] = new_result;
       
       new_result = F(G);
              
       test = bdd_equal(G[i], new_result);
    }
    
/* define result in G[i]  */

/*  (old_result = new_result) or (*steps > max_step)  */

if ( test )
then  /* old_result = new_result  */
    { 
      delete_bdd(new_result); 

      /*  result defined in G[i] */
    }
else  /* old_result != new_result  */
    {
       delete_bdd(G[i]);
       delete_bdd(new_result);
       G[i] = NULL;

      /*  result defined in G[i]  */
    
    }
    

}  /*  end program  */  
    

