
#include "local.h"

#include "htable.h"


/*
******************************
This file contains primitive to manipulate
bdds when using a many variable systems
*******************************
*/
		

#define dbg_flag 0  /* normal value: 0  */

#define X_VAR_SZ 2


bdd compose_x(bdd F, int i, bdd G)
{
  return (compose(F, X_VAR_SZ*i, G));
}


bdd compose_y(bdd F, int i, bdd G)
{
  return (compose(F, X_VAR_SZ*i + 1, G));
}





/*
*********************************
Shift indexes of ALL variables.
Note it si not possible to shift
indexes only for one system
of variables.
*********************************
*/

bdd shift_vars(bdd F, int k)
{
   return (shift_variables(F, X_VAR_SZ*k));
}



bdd exists_x(int i, bdd F)
{
   return (there_exists(X_VAR_SZ*i, F));
}



bdd exists_y(int i, bdd F)
{
   return (there_exists(X_VAR_SZ*i + 1, F));
}

bdd forall_x(int i, bdd F)
{
   return (for_all(X_VAR_SZ*i, F));
}


bdd forall_y(int i, bdd F)
{
   return (for_all(X_VAR_SZ*i + 1, F));
}


bdd restrict_x(bdd F, int i, int rbit)
{
   return (restrict(F, X_VAR_SZ*i, rbit));
}

bdd restrict_y(bdd F, int i, int rbit)
{
   return (restrict(F, X_VAR_SZ*i + 1, rbit));
}


bdd bddvar_x(int i)
{
   return (varx(X_VAR_SZ*i));
}



bdd bddvar_y(int i)
{
   return (varx(X_VAR_SZ*i + 1));
}



bdd ifter_x(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
)
{

  return 
(ifter(G0, G1, G2, X_VAR_SZ*i, rbit, rswitch));
}



bdd ifter_y(
bdd G0, 
bdd G1, 
bdd G2,
unsigned int i,    /*  variable to be restricted  */
unsigned int rbit,   /*  G1, (G2) restricted to rbit (not(rbit)) on var(i)   */
unsigned int rswitch  /* restriction switch: restriction iff rswitch != 0  */
)
{

  return 
(ifter(G0, G1, G2, X_VAR_SZ*i + 1, rbit, rswitch));
}



int is_var_x(bdd G, unsigned int *k)
{
  int i;

  i = is_var(G, k);

  if (i == 0)
  { return (0); }

/* var  */

  if ((*k)%X_VAR_SZ == 0)
  { *k = (*k)/X_VAR_SZ; 
     return (1);
  }
  else { return (0); }

}  /* is_var_x  */ 



int is_var_y(bdd G, unsigned int *k)
{
  int i;

  i = is_var(G, k);

  if (i == 0)
  { return (0); }

/* var  */

  if ((*k)%X_VAR_SZ == 1)
  { *k = ((*k) - 1)/X_VAR_SZ; 
     return (1);
  }
  else { return (0); }

}  /* is_var_y  */ 
