/*
******************************
File: ordsat.c

Define satisfy_all w.r.t ordlist.

Print in satfile
all assignments satisfying
bdd G accordingly to argument ordering.

This version does not print 'x' for dont care variables.
All possible satisfying assignment are printed instead.
*******************************
*/


#include "local.h"
#include <math.h>
#include <limits.h>
#include <float.h>

#define SPLIT 2.1  /* portion of DBL_MAX allowed */


#define dbg_flag 0  /* normal value: 0  */

static void sat_all_step(
FILE *satfile,
unsigned int bddvar_scanidx,
unsigned int *vtxstate,
char *x,                /* bddvar asgs's */
unsigned int maxarg,    /* max number of bdd variables expected */
unsigned int size_ord,  /* size of ord_array */
unsigned int *ord_array
);



static
void goto_next_state(
	unsigned int *oldstate,
	unsigned int *newstate,
	int xvalue
		);


static
void init_args(char *x, unsigned int maxarg);


static
unsigned int arg_from_var(
	unsigned int size_ord, 
	unsigned int *ord_array,
	unsigned int i
		         );


static
unsigned int get_last_arg(
	unsigned int scn_idx, 
	unsigned int size_ord, 
	unsigned int *ord_array
			);




/*
********************
We have
ord_array[i] = argument represented by bdd variable i.
********************
*/



static unsigned int num_x_sofar = 0;

static long double asg_counter = 0.0;




void ord_satisfy_all(
FILE *satfile,
bdd G,
unsigned int maxarg,  /* max number of bdd variables expected */
unsigned int size_ord,  /* size of ord_array */
unsigned int *ord_array
)

{
/*  declarations  */

char *x;
unsigned int vtxstate[5];

unsigned int i = 0;  /* scan ndex  */
/* count satisfying asg  */

/*
vtxstate[0] = vindex;
vtxstate[1] = pflip;
vtxstate[2] = flip;
vtxstate[3] = pswap;
vtxstate[4] = bdd vertex (cast to ptrvertex)
*/


/*  program  */


#if (dbg_flag > 100)
printf("\nord_satisfy_all: begin: maxarg = %u", maxarg);
#endif

fprintf(satfile, "\ninfinity == %E", DBL_MAX);
fprintf(satfile, "\nmax num x's before infinity == %u\n", 
(unsigned int) floor((log(DBL_MAX/SPLIT)/log(2.0)))
);




/* get x array */
x = (char *) malloc(maxarg*sizeof(char));
if ( x == NULL )
/* error */
{ printf("\nord_satisfy_all: error: no malloc space for x\n");
  exit (1);
}

/* ok x */


/* initialize vertex state  */
vtxstate[0] = read_graph_vindex(G);
vtxstate[1] = read_graph_p_flip(G);
vtxstate[2] = read_graph_flip(G);
vtxstate[3] = read_graph_p_swap(G);
vtxstate[4] = (unsigned int) read_graph_root(G);


init_args(x, maxarg);



sat_all_step(
satfile,
0,
vtxstate,
x,             /* bddvar asgs's */
maxarg,
size_ord,
ord_array
);



/* delete x */

free(x);


#if (dbg_flag > 100)
printf("\nord_satisfy_all: ckp 2000 (end)");
#endif


}  /* satisfy_all  */











static void sat_all_step(
FILE *satfile,
unsigned int i,  /*  bddvar_scanidx  */
unsigned int *vtxstate,
char *x,                /* bddvar asgs's */
unsigned int maxarg,    /* max number of bdd variables expected */
unsigned int size_ord,  /* size of ord_array */
unsigned int *ord_array
)
{
/*  declarations  */
unsigned int j;
unsigned int newstate[5];
unsigned int last_arg_idx;


#if (dbg_flag > 0)
static unsigned int count = 0;
#endif 

/*  program  */

#if (dbg_flag > 0)
count++;
#endif

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: begin: bddvar_scanidx = %u (= i), num_x_sofar = %u", 
count, 
i,
num_x_sofar
);
#endif

if ( terminal( (ptrvertex) vtxstate[4] ) )
/*  v terminal, sat asg found */
   {
     if (vtxstate[1] == 0) 
     /*  failure: nonsat asg.  */
       {
#if (dbg_flag > 100)
printf("\nsat_all_step:%u: end: terminal: no sat asg found", count);
#endif        
          return;
       }

     else  /* vtxstate[1] == 1: sat asg found  */
         {
            /* print sat asg  */

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: end: terminal: sat asg found:",
count
);


printf("\nSPLIT = %f, DBL_MAX = %E, DBL_MAX/SPLIT = %E, log10(DBL_MAX/SPLIT) = %E, floor() = %u",
SPLIT,
DBL_MAX ,
DBL_MAX/SPLIT ,
log10(DBL_MAX/SPLIT),
(unsigned int) floor(log10(DBL_MAX/SPLIT))
);

printf("\nlog(DBL_MAX/SPLIT)/log(2.0) = %E, floor(log(DBL_MAX/SPLIT)/log(2.0)) = %u",
log(DBL_MAX/SPLIT)/log(2.0),
(unsigned int) floor(log(DBL_MAX/SPLIT)/log(2.0))
);

printf("\nnum_x_sofar = %u, max allowed num_x_sofar = %u",
num_x_sofar,
(unsigned int) floor(log(DBL_MAX/SPLIT)/log(2.0)) 
);

printf("\nasg_counter = %LG, max allowed asg_counter = %E", 
asg_counter,
(DBL_MAX/SPLIT)
);

#endif


if ( (num_x_sofar < (unsigned int) floor((log(DBL_MAX/SPLIT)/log(2.0))) ) 
     &&
     (asg_counter < (DBL_MAX/SPLIT)) 
  )
   /* ok */
  {

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: asg_counter, num_x_sofar ok",
count
);

printf("; pow = %G",
pow(2.0, (double) num_x_sofar)
);

#endif

             asg_counter = 
asg_counter + 
pow(2.0, (double) num_x_sofar);



  }
  else  /* too big */
  {


#if (dbg_flag > 100)
printf("\nsat_all_step:%u: asg_counter, num_x_sofar too big",
count
);

#endif


asg_counter = DBL_MAX;

  }


      fprintf(satfile, "\nsatasg.%LG: ", asg_counter);


             last_arg_idx = get_last_arg(i, size_ord, ord_array);


for (j = 0; j <= maxarg/2; j++)
                {
      fprintf(satfile, "%c", x[j]);
                }

      fprintf(satfile, "   ");

for (j = maxarg/2 + 1; j < last_arg_idx; j++)
                {
      fprintf(satfile, "%c", x[j]);
                }



#if (dbg_flag > 100)
printf("\nlast_arg_idx = %u: satasg.%LG: ", 
last_arg_idx, 
asg_counter
);

  for (j = 0; j <= maxarg/2; j++)
                {
      printf("%c", x[j]);
                }

      printf("   ");

  for (j = maxarg/2 + 1; j < last_arg_idx; j++)
                {
      printf("%c", x[j]);
                }

#endif

              return;
          }

   }




#if (dbg_flag > 100)
printf("\nsat_all_step:%u: nonterminal vtx", count);
#endif

/*   v nonterminal  */

/*  function may or may not dependent of bdd var i */
/* we use doncares for var with index > maxarg/2  */

/*  we do not use dont cares */


if ( 
        (vtxstate[0] > i)
/*     && ( arg_from_var(size_ord, ord_array, i) > maxarg/2 )  */
   )
/*  function independent of bdd var i */
  {

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: f indp on x[%u], i = %u", count, 
arg_from_var(size_ord, ord_array, i),
i
);

if ( x[arg_from_var(size_ord, ord_array, i)] != 'x' )
  /* error */
  {
fprintf(stderr,
"\nsat_all_step:%u: error: i = %u, x[%u] == '%c' != 'x'", 
count, 
i,
arg_from_var(size_ord, ord_array, i),
x[arg_from_var(size_ord, ord_array, i)]
);

   exit (1); 
  }
#endif

      x[arg_from_var(size_ord, ord_array, i)] = 'x';


      sat_all_step(
satfile, 
i + 1, 
vtxstate, 
x,
maxarg,
size_ord,
ord_array
);
   }
else  /*  function depends on bdd var i  */
   {

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: f dep on x[%u]: begin asg 0: i = %u, num_x_sofar = %u", count, 
arg_from_var(size_ord, ord_array, i),
i,
num_x_sofar
);
#endif


      if ( arg_from_var(size_ord, ord_array, i) <= maxarg/2 )
      num_x_sofar--;


#if (dbg_flag > 100)
printf("\nsat_all_step:%u: x[%u], i = %u, num_x_sofar = %u", 
count, 
arg_from_var(size_ord, ord_array, i),
i,
num_x_sofar
);
#endif


      x[arg_from_var(size_ord, ord_array, i)] = '0';



      /* go to next state  */
      goto_next_state(vtxstate, newstate, 0);

      sat_all_step(
satfile, 
i + 1, 
newstate, 
x,
maxarg,
size_ord,
ord_array
);


#if (dbg_flag > 100)
printf("\nsat_all_step:%u: f  on x[%u]: end asg 0: i = %u", count, 
arg_from_var(size_ord, ord_array, i),
i
);
#endif



#if (dbg_flag > 100)
printf("\nsat_all_step:%u: f  on x[%u]: begin asg 1: i = %u", count, 
arg_from_var(size_ord, ord_array, i),
i
);
#endif

      x[arg_from_var(size_ord, ord_array, i)] = '1';


      /* go to next state  */
      goto_next_state(vtxstate, newstate, 1);


      sat_all_step(
satfile, 
i + 1, 
newstate, 
x,
maxarg,
size_ord,
ord_array
);




#if (dbg_flag > 100)
printf("\nsat_all_step:%u: f  on x[%u]: end asg 1: i = %u", count, 
arg_from_var(size_ord, ord_array, i),
i
);
#endif


/* reset x[] */
      x[arg_from_var(size_ord, ord_array, i)] = 'x';

      if ( arg_from_var(size_ord, ord_array, i) <= maxarg/2 )
      num_x_sofar++;



#if (dbg_flag > 100)
printf("\nsat_all_step:%u: resetted: x[%u] = %c: i = %u", count, 
arg_from_var(size_ord, ord_array, i),
x[arg_from_var(size_ord, ord_array, i)],
i
);
#endif


   }   /*  else  */




}  /*  sat_all_step  */











static
void goto_next_state(
unsigned int *oldstate,
unsigned int *newstate,
int xvalue
)
{

if (
	(   xvalue   +
            oldstate[3]  /* pswap  */ 
        )%2 
	== 0
    )
/* no flip used to update p_flip  */
	{
		/* update pflip  */
newstate[1] = oldstate[1];


		/* update virtual index  */
newstate[0] = 
oldstate[0] +  
read_vertex_shift( (ptrvertex) oldstate[4], 0) + 1;


		/* update pswap  */
newstate[3] = 
(
oldstate[3] +  
read_vertex_swap( (ptrvertex) oldstate[4], 0)
)%2;


		/* update flip  */

newstate[2] = 
read_vertex_flip( (ptrvertex) oldstate[4], 0);

              /* update vertex  */

newstate[4] = 
(unsigned int ) read_vertex_next( 
(ptrvertex) oldstate[4], 
0
);

		return ;

	}

/* side == 1: use flip to update p_flip  */
else
	{

/*  update pflip  */
newstate[1] = (oldstate[1] + oldstate[2])%2;


		/* update virtual index  */


newstate[0] = 
oldstate[0] +  
read_vertex_shift( (ptrvertex) oldstate[4], 1) + 1;



		/* update pswap  */
newstate[3] = 
(
oldstate[3] +  
read_vertex_swap( (ptrvertex) oldstate[4], 1)
)%2;


		/* update flip  */
newstate[2] = 
read_vertex_flip( (ptrvertex) oldstate[4], 1);



              /* update vertex  */

newstate[4] = 
(unsigned int ) read_vertex_next( 
(ptrvertex) oldstate[4], 
1
);

		return ;

	}





}  /* goto_next_state  */





unsigned int get_last_arg(
unsigned int scn_idx, 
unsigned int size_ord, 
unsigned int *ord_array
)
{
/* dcl */
unsigned int result = 0;
unsigned int i;



/* prg */

if (scn_idx >= size_ord)
{

return (scn_idx);

}



/* scn_idx < size_ord */

for (i = 0; i < scn_idx; i++)
   {   /*  for i  */

    if (result < 1 + ord_array[i])
    /* update */
     {  result = 1 + ord_array[i]; }


   }   /*  for i  */

return ( result );


}  /*  get_last_arg      */






void init_args(char *x, unsigned int maxarg)
{
/*  dcl  */
unsigned int i;

/*  prg  */

for (i = 0; i < maxarg; i++)
   {
      x[i] = 'x';
   }


num_x_sofar = maxarg/2 + 1;

asg_counter = 0.0;

}  /*  init_args  */







unsigned int arg_from_var(
	unsigned int size_ord, 
	unsigned int *ord_array,
	unsigned int i
		         )
{
/* dcl */


/* prg  */

if (i < size_ord)
/* in ord_array */
	{ return (ord_array[i]); }
else
	{ return (i); }

}  /* arg_from_var  */









