/*
******************************
File: satisfy.c

Define satisfy_all.

Print in satfile
all assignments satisfying
bdd G.
*******************************
*/


#include "local.h"

#define dbg_flag 0



/* max_arity = max arity of functions involved  */

#define max_arity 2000  


static
void sat_all_step(
FILE *satfile,
unsigned int i,
unsigned int *vtxstate,
unsigned int *asgcount,
char *x
);

static
void goto_next_state(
unsigned int *oldstate,
unsigned int *newstate,
int xvalue
);














void satisfy_all(
FILE *satfile,
bdd G 
)

{
/*  declarations  */


unsigned int vtxstate[5];

unsigned int i = 0;  /* scan ndex  */
unsigned int asgcount = 0; 
/* count satisfying asg  */


char x[max_arity];;

/*
vtxstate[0] = vindex;
vtxstate[1] = pflip;
vtxstate[2] = flip;
vtxstate[3] = pswap;
vtxstate[4] = bdd vertex (cast to ptrvertex)
*/


/*  program  */


#if (dbg_flag > 100)
printf("\nsatisfy_all: ckp 100 (begin)");
#endif

/* initialize vertex state  */
vtxstate[0] = read_graph_vindex(G);
vtxstate[1] = read_graph_p_flip(G);
vtxstate[2] = read_graph_flip(G);
vtxstate[3] = read_graph_p_swap(G);
vtxstate[4] = (unsigned int) read_graph_root(G);


sat_all_step(
satfile,
i,
vtxstate,
&asgcount,
x
);

#if (dbg_flag > 100)
printf("\nsatisfy_all: ckp 2000 (end)");
#endif


}  /* satisfy_all  */











static
void sat_all_step(
FILE *satfile,
unsigned int i,
unsigned int *vtxstate,
unsigned int *asgcount,
char *x
)
{
/*  declarations  */
unsigned int j;
unsigned int newstate[5];

#if (dbg_flag > 0)
static unsigned int count = 0;
#endif 

/*  program  */

#if (dbg_flag > 0)
count++;
#endif

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 100 (begin)", count);
#endif

if ( terminal( (ptrvertex) vtxstate[4] ) )
/*  v terminal, sat asg found */
   {
     if (vtxstate[1] == 0) 
     /*  failure: nonsat asg.  */
       {
#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 200 (end: no sat asg)", count);
#endif        
          return;
       }

     else  /* vtxstate[1] == 1: sat asg found  */
         {
            /* print sat asg  */

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 300 (end: sat asg found)", count);
#endif

      fprintf(satfile, "\n\nsatasg.%u: ", *asgcount);
             for (j = 0; j < i; j++)
                {
      fprintf(satfile, "%c", x[j]);
                }
             *asgcount = (*asgcount) + 1;
              return;
          }

   }


#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 400 (nonterminal vtx)", count);
#endif

/*   v nonterminal  */

if (vtxstate[0] > i)
/*  function independent of x[i]  */
  {

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 500 (f indp on x[%u])", count, i);
#endif

      x[i] = 'x';
      sat_all_step(
satfile, 
i + 1, 
vtxstate, 
asgcount, 
x
);
   }
else  /*  function depends on x[i]  */
   {

#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 600 (f dep. on x[%u]: 0)", count, i);
#endif

      x[i] = '0';
      /* go to next state  */
      goto_next_state(vtxstate, newstate, 0);

#if (dbg_flag > 150)
printf("\nsat_all_step:%u: ckp 620 (f dep. on x[%u]: 0)", count, i);
#endif

      sat_all_step(
satfile, 
i + 1, 
newstate, 
asgcount, 
x
);



#if (dbg_flag > 100)
printf("\nsat_all_step:%u: ckp 700 (f dep on x[%u]: 1)", count, i);
#endif

      x[i] = '1';
      /* go to next state  */
      goto_next_state(vtxstate, newstate, 1);

#if (dbg_flag > 150)
printf("\nsat_all_step:%u: ckp 720 (f dep on x[%u]: 1)", count, i);
#endif

      sat_all_step(
satfile, 
i + 1, 
newstate, 
asgcount, 
x
);
   }



}  /*  sat_all_step  */











static
void goto_next_state(
unsigned int *oldstate,
unsigned int *newstate,
int xvalue
)
{

if (
	(   xvalue   +
            oldstate[3]  /* pswap  */ 
        )%2 
	== 0
    )
/* no flip used to update p_flip  */
	{
		/* update pflip  */
newstate[1] = oldstate[1];


		/* update virtual index  */
newstate[0] = 
oldstate[0] +  
read_vertex_shift( (ptrvertex) oldstate[4], 0) + 1;


		/* update pswap  */
newstate[3] = 
(
oldstate[3] +  
read_vertex_swap( (ptrvertex) oldstate[4], 0)
)%2;


		/* update flip  */

newstate[2] = 
read_vertex_flip( (ptrvertex) oldstate[4], 0);

              /* update vertex  */

newstate[4] = 
(unsigned int ) read_vertex_next( 
(ptrvertex) oldstate[4], 
0
);

		return ;

	}

/* side == 1: use flip to update p_flip  */
else
	{

/*  update pflip  */
newstate[1] = (oldstate[1] + oldstate[2])%2;


		/* update virtual index  */


newstate[0] = 
oldstate[0] +  
read_vertex_shift( (ptrvertex) oldstate[4], 1) + 1;



		/* update pswap  */
newstate[3] = 
(
oldstate[3] +  
read_vertex_swap( (ptrvertex) oldstate[4], 1)
)%2;


		/* update flip  */
newstate[2] = 
read_vertex_flip( (ptrvertex) oldstate[4], 1);



              /* update vertex  */

newstate[4] = 
(unsigned int ) read_vertex_next( 
(ptrvertex) oldstate[4], 
1
);

		return ;

	}





}  /* goto_next_state  */



























