/*
******************************
File: bddtoc.c

Define C procedure implementing
given BDD.


Print in satfile.
*******************************
*/


#include "local.h"

#define dbg_flag 0


typedef unsigned int ht_value;


struct Hash_Table
{
  unsigned int size_table;
  unsigned int elements_in_table;
  unsigned int non_null_entry;
  ht_value *table;
};


typedef struct Hash_Table hash_table;





unsigned int *get_new_bk();
unsigned int *read_bk_next(unsigned int *v);
unsigned int read_bk_value(unsigned int *v);
unsigned int read_bk_key(unsigned int *v);


static htable T;   /* to be used as markers  */

static void storeinbddtab(ptrvertex v);

static unsigned int store_bddvtx(ptrvertex v);





void bdd2c(
	FILE *pfile, 
	bdd G, 
	unsigned int maxarg,
	unsigned int size_ord, 
	unsigned int *ord_array,
        char *fname
)
{

/* declarations   */
ptrvertex v, term_vtx;
unsigned int i;
unsigned int value, term_vtx_name;
static int ck_calls = 0;
FILE *hfile;
FILE *glbfile;



/*  prg  */

if (ck_calls == 0)
/* first call */
{

#if 0
glbfile = fopen("bddvtx2c.mkvtx.1", w);
fprintf(glbfile, "/* mkvtx calls */\n\n");
fclose(glbfile);
#endif



fprintf(pfile, "\n#include \"bddvtx2c.h\"\n\n");

/* in glbfile  */

glbfile = fopen("bddvtx2c.glb.c", "w");

fprintf(glbfile, "\n#include \"bddvtx2c.h\"\n\n");

fprintf(glbfile,
"unsigned int size_ord = %u;\n",
size_ord
);


/*  size_ord > 0  */

fprintf(glbfile,
"unsigned int ord_array[%u] = {%u", size_ord, ord_array[0]);

for (i = 1; i < size_ord; i++)
   { fprintf(glbfile, ", %u", ord_array[i]); }

fprintf(glbfile, "};\n\n");



T = get_htable();

term_vtx = get_terminal_bddvtx();
storeinbddtab(term_vtx);
read_ht(T, (unsigned int) term_vtx, &term_vtx_name);


/* term_vtx_name  */
fprintf(glbfile, "\n\nunsigned int term_vtx_name = %u;\n\n",
term_vtx_name
);


fprintf(glbfile, "\n\nstruct bdd2cvtx *bddvtxtab;\n\n");


fclose(glbfile);


/* in .h  */

hfile = fopen("bddvtx2c.h", "w");

fprintf(hfile, "/* file: bddvtx2c.h  */\n\n");
fprintf(hfile, "#ifndef bddvtx2c_h\n");
fprintf(hfile, "#define bddvtx2c_h\n\n\n");
fprintf(hfile, "#include <stdlib.h>\n");
fprintf(hfile, "#include <stdio.h>\n\n");

fprintf(hfile, "#include \"bddvtx2c.init.h\"\n\n");

fprintf(hfile, "\n\nvoid init_bdd2c_env();");

fprintf(hfile, "\n\nint bdd2c_eval(unsigned int vtxname, unsigned int vindex, \n\
char p_flip, char flip, char p_swap, char *x);\n\n"
);


fprintf(hfile, "extern unsigned int size_ord;\n");

fprintf(hfile,
"extern unsigned int ord_array[%u];", size_ord);


fprintf(hfile, "\n\nextern unsigned int term_vtx_name;\n\n");

/* bdd2cvtx  */
fprintf(hfile, "\n\nstruct bdd2cvtx\n");
fprintf(hfile, "{");
fprintf(hfile, "\n\tunsigned int vtx_next[2];");
fprintf(hfile, "\n\tunsigned int shift[2];");
fprintf(hfile, "\n\tchar flip[2];");
fprintf(hfile, "\n\tchar swap[2];");
fprintf(hfile, "\n};\n");

fprintf(hfile, "\n\nextern struct bdd2cvtx *bddvtxtab;\n\n");

fprintf(hfile, "\n\nvoid mkvtx(\n");
fprintf(hfile, "unsigned int vtxname,\n");
fprintf(hfile, "unsigned int next0,\n");
fprintf(hfile, "unsigned int next1,\n");
fprintf(hfile, "unsigned int shift0,\n");
fprintf(hfile, "unsigned int shift1,\n");
fprintf(hfile, "unsigned int swap0,\n");
fprintf(hfile, "unsigned int swap1,\n");
fprintf(hfile, "unsigned int flip0,\n");
fprintf(hfile, "unsigned int flip1\n");
fprintf(hfile, ");\n\n\n");

fclose(hfile);




ck_calls++;

}




/* ck_calls > 0  */


storeinbddtab(read_graph_root(G));
read_ht(T, (unsigned int) read_graph_root(G), &value);


hfile = fopen("bddvtx2c.h", "a");

fprintf(hfile, "#define %s(x) bdd2c_eval(%u, %u, %u, %u, %u, x) /* arity <= %u */\n", 
fname, 
value,
read_graph_vindex(G),
read_graph_p_flip(G),
read_graph_flip(G),
read_graph_p_swap(G),
maxarg
);

fclose(hfile);

}




void storeinbddtab(ptrvertex v)
{
/* dcls */
unsigned int value;


/*  prg  */

/* visit v  */

store_bddvtx(v);



if ( nonterminal(v) )
	{  /* visit nonvisited v sons  */
		
          if ( ! read_ht(T, 
                   (unsigned int) read_vertex_next(v, 0),
                   &value)
          	)
          /* nonvisited  */  
          storeinbddtab(read_vertex_next(v, 0));
          
          
          if ( ! read_ht(T, 
          		(unsigned int) read_vertex_next(v, 1),
          		&value)
          	)
          /* nonvisited  */  
          storeinbddtab(read_vertex_next(v, 1));
	}




}  /*  storeinbddtab  */ 










unsigned int store_bddvtx(ptrvertex v)
{
/* dcls */
static unsigned int vtx_count = 0;
unsigned int value;

/* prg */

if ( ! read_ht(T, (unsigned int) v, &value) )
/* v nonvisited */
{
/* store v in bddvtxtable and lock v */

vtx_count++;
write_ht(T, (unsigned int) v, vtx_count);

lock_bdd_vertex(v);

return (vtx_count);

}


/* v already in T */

return (value);

}  /*  store_bddvtx  */ 







/*
Print T and unlock vertices
*/


void dumpbddtab(FILE *myfile)
{
/* dcls */
unsigned int i;
ht_value *v;
FILE *hfile;

unsigned int value0, value1;

FILE *auxfile;

/* prg */



fprintf(myfile, "\n\n\nvoid init_bdd2c_env()\n {\n");


fprintf(
myfile, 
"\nbddvtxtab = (struct bdd2cvtx *) malloc(sizeof(struct bdd2cvtx)*%u);\n\n",
(get_htable_elements(T) + 1)
);


fprintf(myfile, "\n\n\n#include \"bddvtx2c.init\"\n\n}\n\n");

auxfile = fopen("bddvtx2c.init.code", "w");
fprintf(auxfile, "/* env functions */\n\n");
fprintf(auxfile, "init_bdd2c_env_1();\n\n");
fclose(auxfile);


auxfile = fopen("bddvtx2c.init.h", "w");
fprintf(auxfile, "/* dcl env functions */\n\n");
fprintf(auxfile, "void init_bdd2c_env_1();\n\n");
fclose(auxfile);

auxfile = fopen("bddvtx2c.init.1.c", "w");
fprintf(auxfile, "#include \"bddvtx2c.h\"\n\n");
fprintf(auxfile, "void init_bdd2c_env_1()\n {\n\n");
fprintf(auxfile, "#include \"bddvtx2c.mkvtx.1\"\n\n");
fprintf(auxfile, "}\n\n");
fclose(auxfile);



auxfile = fopen("bddvtx2c.mkvtx.1", "w");



    for (i = 0; i < get_htable_size(T); i++)
       {
          v = (ht_value *) ( ((hash_table *) T) -> table[i] );
          
          while (v != NULL)
          {
             
if (
( (! read_ht(
T, 
(unsigned int) read_vertex_next((ptrvertex) read_bk_key(v), 0), 
&value0
          )
  )
  ||
 (! read_ht(
T, 
(unsigned int) read_vertex_next((ptrvertex) read_bk_key(v), 1), 
&value1
          )
  )
)
&&
nonterminal((ptrvertex) read_bk_key(v))
   )
/* warning ??? */
{ 
fprintf(auxfile, "/* warning: vtx = 0x%x, next0 = 0x%x, next1 = 0x%x */ ",
read_bk_key(v),
read_vertex_next((ptrvertex) read_bk_key(v), 0),
read_vertex_next((ptrvertex) read_bk_key(v), 1)
); 
}


fprintf(auxfile, "mkvtx(%u, %u, %u, %u, %u, %u, %u, %u, %u);\n",
read_bk_value(v),
value0,
value1,
read_vertex_shift((ptrvertex) read_bk_key(v), 0),
read_vertex_shift((ptrvertex) read_bk_key(v), 1),
read_vertex_swap((ptrvertex) read_bk_key(v), 0),
read_vertex_swap((ptrvertex) read_bk_key(v), 1),
read_vertex_flip((ptrvertex) read_bk_key(v), 0),
read_vertex_flip((ptrvertex) read_bk_key(v), 1)
);

lock_bdd_vertex((ptrvertex) read_bk_key(v));

             v = read_bk_next(v);
             
           }
          
       }



delete_ht(T);

fclose(auxfile);


hfile = fopen("bddvtx2c.h", "a");
fprintf(hfile, "\n\n#endif\n\n");
fclose(hfile);

 

}  /*  dumpbddtab  */






