/*
***********************************************
file: test.c

Test for bdd vertex procedures.
***********************************************
*/



/*
**************************************************
B is as follows:
B[0] = flip side 0;
B[1] = flip side 1;
B[2] = swap side 0;
B[3] = swap side 1;
B[4] = shift side 0;
B[5] = shift side 1;
B[6] = next side 0;
B[7] = next side 1;
**************************************************
*/



#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "bddvtx.h"
#include "dag.h"
#include "gvertex.h"

#define print_on_file 1000
#define dbg_frexp 0




ptrvertex test_bddvtx(unsigned int *A, unsigned int *V);
void test2_bddvtx(unsigned int *A, ptrvertex old_v);


main()
{

/* declarations  */
unsigned int i;

#if 1
int j;
double frpq;
double x = 17.0;
#endif

unsigned int V[10];
unsigned int A[10];
ptrvertex v;


/*
****************************
begin:    output redirection   
****************************
*/

#if (print_on_file > 0)
fclose(stdout);
freopen("test.out", "w", stdout);
#endif


/*
****************************
end:    output redirection   
****************************
*/
setbuf(stdout, NULL);



/* program  */

/* start dag environment   */


printf(
"\nTest 0 frexp: frexp = 0x%x; (* freexp) = 0x%x",
(unsigned int) frexp,
(unsigned int) (*frexp)
);
printf("\nx = %f; &j = 0x%x",
x,
&j
);

frpq = frexp(x, &j);

printf("\nfrexp(x, &j) = %f; j = %d",
frpq,
j
);



#if (dbg_frexp > 0)
testfrexp();
#endif


start_dag_env();

#if (dbg_frexp > 0)
testfrexp();
#endif


start_bddvtx_env();  

#if (dbg_frexp > 0)
testfrexp();
#endif


/*  tests   */
for (i = 0; i < 10; i++)
	{
		V[i] = (unsigned int) NULL;
	}

#if (dbg_frexp > 0)
testfrexp();
#endif

for (i = 0; i < 20; i++)
	{
		v = test_bddvtx(A, V);

#if 0
		test2_bddvtx(A, v);
#endif

	}


}   /* main  */




ptrvertex test_bddvtx(unsigned int *A, unsigned int *V)
{
/* declarations  */
ptrvertex v;

static unsigned int j = 0;

unsigned int i;


unsigned int 
	flip_0,
	swap_0,
	flip_1,
	swap_1,
	shift_0,
	shift_1,
	next_0,
	next_1;

/* program  */

j++;
printf("\n\nTest %u bdd vertex", j);

/* define vertex labels   */

flip_0 = rand()%2;
flip_1 = rand()%2;
swap_0 = rand()%2;
swap_1 = rand()%2;
shift_0 = 16 + rand()%2;
shift_1 = 16 + rand()%2;
next_0 = V[rand()%10];
next_1 = V[rand()%10];


A[0] = flip_0;
A[1] = flip_1;
A[2] = swap_0;
A[3] = swap_1;
A[4] = shift_0;
A[5] = shift_1;
A[6] = next_0;
A[7] = next_1;

v = get_bdd_vertex(A);

/* store v  */
V[rand()%10] = (unsigned int) v;


print_gvtx(v);
printf("\nsize_gvtx_nonptrlab(0x%x) = %u", 
v, size_gvtx_nonptrlab(v));
printf("\nsize_gvtx_ptrlab(0x%x) = %u", 
v, size_gvtx_ptrlab(v));
printf("\nflip_0 = %u = %u = read_vertex_flip(v, 0)",
flip_0, read_vertex_flip(v, 0)
);
printf("\nflip_1 = %u = %u = read_vertex_flip(v, 1)",
flip_1, read_vertex_flip(v, 1)
);
printf("\nswap_0 = %u = %u = read_vertex_swap(v, 0)",
swap_0, read_vertex_swap(v, 0)
);
printf("\nswap_1 = %u = %u = read_vertex_swap(v, 1)",
swap_1, read_vertex_swap(v, 1)
);
printf("\nshift_0 = %u = %u = read_vertex_shift(v, 0)",
shift_0, read_vertex_shift(v, 0)
);
printf("\nshift_1 = %u = %u = read_vertex_shift(v, 1)",
shift_1, read_vertex_shift(v, 1)
);
printf("\nnext_0 = 0x%x = 0x%x = read_vertex_next(v, 0)",
next_0, read_vertex_next(v, 0)
);
printf("\nnext_1 = 0x%x = 0x%x = read_vertex_next(v, 1)",
next_1, read_vertex_next(v, 1)
);

lock_bdd_vertex(v);
unlock_bdd_vertex(v);

return (v);

}





void test2_bddvtx(unsigned int *A, ptrvertex old_v)
{
/* declarations  */
ptrvertex v;

static unsigned int j = 0;

/* program  */

j++;

printf("\nTest_2: %u bdd vertex", j);

v = get_bdd_vertex(A);

/* test v  */

printf("\nold_v = 0x%x = 0x%x = new_v", 
old_v, v);

if (v != old_v)
{
printf("\nERROR: old_v != new_v"); 
fprint_dag_gvtx(stdout);
exit (1000);
}


}

