/*
*************************************
file: upload.c

Load program in file into RAM.
*************************************
*/


#include "local.h"

#include "runner.h"
#include "lextkns.h"
#include "prgvtx.h"
#include "apply.h"
#include "bdd.h"
#include <time.h>


/* normal value: 10, use 0 for quiet operation   */
#define print_info 10 

#define ck_bddvalue 0  /* normal value: 0   */


#define dbg_flag 0   /* normal value: 0  */
 






prghdr* upload(char *filename, char *answfile)
{

/* dcl  */

FILE  *lexinput, *tmp_file, *qfile;

prghdr* p;  /*  program header  */

void 	*tail_prg, *end_pad,
	*tail_ordlist,
	*bsaterm, *bsa_vtx;

#if (ck_bddvalue > 0)
static unsigned int i_step = 0;
#endif


#if (print_info > 0)
long time_rhs[2];
#endif


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif



/* prg  */


#if (dbg_flag > 10)
printf("\nupload: begin: open input file, start lexan");
#endif



/* initialize lexan  */

lexinput = fopen(filename, "r");
begin_lexan();
set_lexan(lexinput, filename);




#if (dbg_flag > 10)
printf("\nupload: init. prg. hdr.");
#endif


/* create program header  */
p = get_prghdr();


/* 
  Read maxarg
*/

tmp_file = fopen(".tmp.bsp", "r");
fscanf(tmp_file, "maxarg %u", &(p -> maxarg));
fclose(tmp_file);


#if (dbg_flag > 100)
fprintf(stderr, "\nupload: maxarg = %u", p -> maxarg);
#endif



/* init. answers file */

p -> answfilename = answfile; 
qfile = fopen(answfile, "w");
fprintf(qfile, 
    "Answers to queries in program %s\n\n\n", 
    filename
       );
fclose(qfile);


/* init C file name */
p -> cfilename = NULL; 



/* init ordlist, etc */

p -> ordlist = NULL;

tail_prg = NULL;
tail_ordlist = NULL;



#if (dbg_flag > 10)
printf("\nupload: pad prg with begin");
#endif

p -> prg = get_prgvtx(1);
regeval(p -> prg) = get_prgvtx(size_begin);
vtxeval(regeval(p -> prg), ptr_up) = NULL;
vtxeval(regeval(p -> prg), ptr_down) = NULL;
vtxeval(regeval(p -> prg), op_code) = (void *) tk_begin;
tail_prg = p -> prg;


/* 
   p -> prg != NULL 
   tail_prg != NULL
*/


#if (dbg_flag > 10)
printf("\nupload: no more terms: pad with end");
#endif


/* pad with end */
/* tail_prg != NULL */
vtxeval(regeval(tail_prg), ptr_down) = get_prgvtx(1);
bsa_vtx = get_prgvtx(size_end);
regeval(vtxeval(regeval(tail_prg), ptr_down)) = bsa_vtx;

vtxeval(bsa_vtx, ptr_up) = tail_prg;
vtxeval(bsa_vtx, ptr_down) = NULL;
vtxeval(bsa_vtx, op_code) = (void *) tk_end;
end_pad = vtxeval(regeval(tail_prg), ptr_down); 


/* note: tail_prg always points to instruction before end */
/* This allows for freeing of current instruction */

tail_prg = regfield(end_pad, ptr_up);


#if (dbg_flag > 10)
printf("\nupload: end padding");
#endif



#if (dbg_flag > 10)
printf("\nupload: init timer");
#endif



#if (print_info > 5)
printf("\n\n\n");
#endif


/* initialize timer */
glb_time_rhs[0] = clock();


#if (dbg_flag > 10)
printf("\nupload: init: p -> old_bddvalue");
#endif


/* init old_bddvalue  */
p -> old_bddvalue = glb_bdd_zero;


#if (dbg_flag > 10)
printf("\nupload: read and execute terms");
#endif

/* read and exec all terms in program  */ 

while ( (bsaterm = read_new_term()) != NULL)
     {
       /* bsaterm != NULL */
    
/* redefine tail_prg since, because of free's,
   last instruction may have changed
*/


#if (dbg_flag > 10)
printf("\nupload: bsaterm = 0x%x, old tail_prg = 0x%x, tail_ordlist = 0x%x", bsaterm,
tail_prg,
tail_ordlist
);
#endif

tail_prg = regfield(end_pad, ptr_up);


#if (dbg_flag > 10)
printf("\nupload: new tail_prg = 0x%x", tail_prg);
#endif


/* move to prgvtx  */

bsa_vtx =  *((void* *) bsaterm);

#if (dbg_flag > 10)
printf("\nupload: bsa_vtx = 0x%x", bsa_vtx);
#endif


#if (dbg_flag > 10)
printf("\nupload: ((void* *) bsa_vtx)[op_code] == %u == '%c'", 
((void* *) bsa_vtx)[op_code],
((void* *) bsa_vtx)[op_code]
);
#endif

switch ( (unsigned int) ((void* *) bsa_vtx)[op_code] )
      {
           case tk_ifte:
           case tk_compose:
           case tk_boolean:
           case tk_join:
           case tk_prj:


           case tk_loopback:
           case tk_free:

           case tk_isone:  /* query isone  */
           case tk_eval: /* eval  */
           case tk_satasg:  /* satasg  */
           case tk_C:  /* C function */

           case tk_exec:

                        /* program instruction and queries
                            as well as definition 
                       */


#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_ifte ... tail_prg = 0x%x",
tail_prg
);
printf("\n(*((void** *) tail_prg)) == 0x%x",
(*((void** *) tail_prg))
);
printf("\n(*((void** *) tail_prg))[ptr_down] == 0x%x",
(*((void** *) tail_prg))[ptr_down]
);

#endif

/* tail_prg != NULL */

(*((void** *) tail_prg))[ptr_down] = bsaterm; 

#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_ifte ... ptr_down defined");
#endif


((void* *) bsa_vtx)[ptr_up] = tail_prg; 

#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_ifte ... ptr_up defined");
#endif

((void* *) bsa_vtx)[ptr_down] = end_pad; 
  
#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_ifte ... padding done");
#endif

/* last instruction in program */
tail_prg = bsaterm;

#if (dbg_flag > 10)
printf("\nupload: switch: end tk_ifte ... tail_prg = 0x%x",
tail_prg
);
#endif


                        break;




           case tk_xarg:


                       /* program instruction 
                            as well as definition 
                            note: maxarg already well defined
                       */

#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_xarg ... tail_prg = 0x%x",
tail_prg
);
#endif


/* tail_prg != NULL */

(*((void** *) tail_prg))[ptr_down] = bsaterm; 


((void* *) bsa_vtx)[ptr_up] = tail_prg; 
((void* *) bsa_vtx)[ptr_down] = end_pad; 
  

/* last instruction in program */
tail_prg = bsaterm;


#if 0
if ( 1 + (unsigned int) regfield(tail_prg, xarg_word0) 
      >  
     (p -> maxarg)
   )
/* update maxarg */
  {

p -> maxarg = 
     1 + (unsigned int) regfield(tail_prg, xarg_word0);

  }
#endif

#if (dbg_flag > 10)
printf("\nupload: switch: end tk_xarg ... tail_prg = 0x%x",
tail_prg
);
#endif


                        break;





            case tk_ordlist:  

/* ordlist: not a prg instr. nor a def. */


#if (dbg_flag > 10)
printf("\nupload: switch: begin tk_ordlist ... tail_ordlist = 0x%x",
tail_ordlist
);
#endif



if (tail_ordlist == NULL)
     p -> ordlist = bsaterm;
else (*((void** *) tail_ordlist))[ptr_down] = bsaterm; 


((void* *) bsa_vtx)[ptr_up] = tail_ordlist; 
((void* *) bsa_vtx)[ptr_down] = NULL; 
  

/* last instruction in ordlist list */
tail_ordlist = bsaterm;



#if (dbg_flag > 10)
printf("\nupload: switch: end tk_ordlist ... tail_ordlist = 0x%x",
tail_ordlist
);
#endif

                      break;




           default:  /* error  */

#if (dbg_flag > 10)
printf("\nupload: switch: default ...");
#endif

                     printf("\nError: unknown instruction\n");
                     exit (1000);

      }  /* switch */


#if (dbg_flag > 10)
printf("\nupload: end switch: adjust end_pad ptr_up: tail_prg = 0x%x",
tail_prg
);
#endif

regfield(end_pad, ptr_up) = tail_prg;


/* last (non end) instruction in program */

p -> pc = tail_prg;

#if (dbg_flag > 10)
printf("\nupload: begin apply while: p -> pc == tail_prg == 0x%x\n",  p -> pc);
#endif

while (p -> pc != NULL)
   {

#if 0
#if (print_info > 5)
time_rhs[0] = clock();
printf("%u: pc = %x", ite_counter, p -> pc);
#endif
#endif

#if (dbg_flag > 10)
printf("upload: before apply: p -> pc = 0x%x\n",  p -> pc);
#endif




#if (ck_bddvalue > 10)
fprintf(stderr, "\n\n******begin step %u: old_bddvalue == 0x%x, new_bddvalue == 0x%x\n",
i_step,
p -> old_bddvalue,
p -> new_bddvalue
);
#endif

       apply(regfield(p -> pc, op_code))(p);

#if (ck_bddvalue > 10)
fprintf(stderr, "------end step %u: old_bddvalue == 0x%x, new_bddvalue == 0x%x\n",
i_step,
p -> old_bddvalue,
p -> new_bddvalue
);

i_step++;
#endif




#if (dbg_flag > 10)
printf("upload: after apply: p -> pc = 0x%x\n",  p -> pc);
#endif

#if (dbg_flag > 5)
/* check p -> old_bddvalue */
printf("\nupload: p ->pc == 0x%x, p -> old_bddvalue == 0x%x\n",  
p -> pc,
p -> old_bddvalue
);
if (p -> old_bddvalue == NULL)
  /* error */
  {
    fprintf(stderr, "\nupload: error: p -> old_bddvalue == NULL");
    exit (1);
  }

#endif



#if 0
#if (print_info > 5)
time_rhs[1] = clock();
printf(": %lf (sec.)\n",
((double) (time_rhs[1] - time_rhs[0]))/((double) CLOCKS_PER_SEC));
#endif
#endif

        }   /*  while (p -> pc != NULL)  */

#if (dbg_flag > 10)
printf("upload: end apply while: p -> pc = 0x%x\n",  p -> pc);
#endif


     }  /* while  */



#if (dbg_flag > 10)
printf("\nupload: no more terms: pad with end");
#endif




#if 0
/* False: this needs not to be the case */
#if (dbg_flag > 10)
printf("\nupload: check that size_ordlist == maxarg ");
#endif

if ( p -> maxarg 
    != 
    (unsigned int) regfield(p -> ordlist, ordlist_word0) 
   )
/* error  */
{
 printf("\nupload: error: maxarg = %u != %u = ordlist_size",
          p -> maxarg, regfield(p -> ordlist, ordlist_word0));

  exit (1);
          

}
#endif



#if (dbg_flag > 10)
printf("\nupload: end of bdd computation");
#endif



/*  print bdd, time and memory info  */

print_bdd_info();



#if (dbg_flag > 10)
printf("\nupload: print symbol table info");
#endif

printf("\n\n\nsize symbol table: %u",
get_symbtab_size(Key_Words_Table)
);

printf("\nelements in symbol table now: %u",
get_symbtab_elements(Key_Words_Table)
);

printf("\nmax num elements attained in symbol table: %u\n\n",
get_st_maxsize(Key_Words_Table)
);



#if (dbg_flag > 10)
printf("\nupload: close lexan");
#endif


end_lexan();



#if (dbg_flag > 10)
printf("\nupload: close input file");
#endif


fclose(lexinput);

#if (dbg_flag > 10)
printf("\nupload: return prg.hdr 0x%x", p);
#endif



return (p);


}  /*  prg  */





















