/*
*************************************
file: call.c

Primitives for call analysis.


Primes: 101 30011
*************************************
*/


#include "local.h"
#include "symbtab.h"



#define dbg_flag 0  /* normal value: 0  */
#define CACHE_SIZE 30011   /* this should be a prime number  */
#define KEY_SIZE 25  

/* dbg flag for def1step_calls()  */
#define dbg1 0   /* normal value: 0  */


/* dbg flag for define_calls()   */
#define dbg2 0  /* normal value: 0 */



/* Cache for calls */
static void *Call_Cache;


static char* make_call_key(void *calling_reg, void *called_reg);
static void store_in_cache(char *key, unsigned int value);



static void update_step(
	prghdr* p, 
	void* head_reg, 
	void* middle_reg, 
	void* tail_reg, 
	int *update_found);


static int update_ifte(prghdr* p, void* calling_reg);
static int update_compose(prghdr* p, void* calling_reg);

static int update_prj(prghdr* p, void* calling_reg);
static int update_join(prghdr* p, void* calling_reg);





static void def1step_calls(prghdr* p);
static void update1step(
	prghdr* p, 
	void* lhs_reg, 
	void* rhs_reg);

static void ifte1step(prghdr* p, void* calling_reg);
static void compose1step(prghdr* p, void* calling_reg);

static void prj1step(prghdr* p, void* calling_reg);
static void join1step(prghdr* p, void* calling_reg);






void start_call_env()
{

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nstart_call_env:%u: begin", count);
#endif

  Call_Cache = create_symb_table(CACHE_SIZE);

#if (dbg_flag > 100)
printf("\nstart_call_env:%u: end: created cache 0x%x", count, Call_Cache);
#endif

}


void end_call_env()
{
#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nend_call_env:%u: begin: delete cache 0x%x", count, Call_Cache);
#endif

delete_symbtab(Call_Cache);

#if (dbg_flag > 100)
printf("\nend_call_env:%u: end", count);
#endif

}















void define_calls(prghdr* p)
{
/* dcl */
void* prg_reg;
void* prg_vtx;

int update_found = 1;


#if (dbg2 > 0)
static int count = 0;
unsigned int ii1 = 0;
unsigned int ii2 = 0;
unsigned int ii3 = 0;
count++;
#endif


/* prg  */

#if (dbg2 > 100)
printf("\ndefine_calls:%u: begin: program = 0x%x", count, p);
#endif


/* define one step calls  */
def1step_calls(p);


#if (dbg2 > 100)
printf("\ndefine_calls:%u: 1 step calls defined", count);
#endif



/* transitive closure  */



while ( update_found )
 {

#if (dbg2 > 100)
ii1++;
ii3 = 0;
printf("\ndefine_calls:%u: begin while update_found: ii1 = %u", 
count, ii1
);
#endif

update_found = 0;

/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);


 while (prg_reg != NULL)
  {

#if (dbg2 > 100)
ii2++;
ii3++;
printf("\ndefine_calls:%u: begin while prg_reg = 0x%x: ", 
count, 
prg_reg
);
printf("ii1 = %u, ii2 = %u. ii3 = %u", ii1, ii2, ii3);
#endif

       /*   prg_reg != NULL */

       prg_vtx = regeval(prg_reg);

/* note that are no xarg, boole instr here */

       switch( (int) vtxeval(prg_vtx, op_code) )
          {


           case tk_ifte:

                    update_found = update_found || update_ifte(p, prg_reg);

#if (dbg2 > 100)
printf("\ndefine_calls:%u: ifte: update_found = %d: ", 
count, update_found
);
printf("ii1 = %u, ii2 = %u. ii3 = %u", ii1, ii2, ii3);
#endif


                     break;



           case tk_compose:
 
                     update_found = update_found || update_compose(p, prg_reg);

#if (dbg2 > 100)
printf("\ndefine_calls:%u: compose: update_found = %d: ", 
count, update_found
);
printf("ii1 = %u, ii2 = %u. ii3 = %u", ii1, ii2, ii3);
#endif

                     break;





           case tk_prj:

                    update_found = update_found || update_prj(p, prg_reg);

#if (dbg2 > 100)
printf("\ndefine_calls:%u: prj: update_found = %d: ", 
count, update_found
);
#endif


                     break;






           case tk_join:

                    update_found = update_found || update_join(p, prg_reg);

#if (dbg2 > 100)
printf("\ndefine_calls:%u: join: update_found = %d: ", 
count, update_found
);
#endif


                     break;



           default:  /* do nothing on query, xarg, boole */
          
                     break;

          }




       prg_reg = vtxeval(prg_vtx, ptr_down);


  }   /* while */


}  /*  while */



#if (dbg2 > 100)
printf("\ndefine_calls:%u: end: program = 0x%x: ", 
count, 
p
);
printf("ii1 = %u, ii2 = %u. ii3 = %u", ii1, ii2, ii3);
#endif


}   /* define_calls  */













int update_ifte(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
int update_found = 0;

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupdate_ifte:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);

 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\nupdate_ifte:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif

       /*   prg_reg != NULL */

for (i = 0; i < 3; i++)
   {

update_step(
	p, 
	calling_reg, 
	regfield(calling_reg, ifte_word0 + i), 
	prg_reg, 
	&update_found);

   }  /*  for i */




prg_reg = regfield(prg_reg, ptr_down);

  }   /*  while  */



#if (dbg_flag > 100)
printf("\nupdate_ifte:%u: end: return %d", count, update_found);
#endif


return ( update_found );

}   /* update_ifte  */








void update_step(
	prghdr* p, 
	void* head_reg, 
	void* middle_reg, 
	void* tail_reg, 
	int *update_found
)
{
/*  dcl */
char *key_tail, *key_all;


void *bucket_tail, *bucket_all;

/* prg */

key_tail = make_call_key(middle_reg, tail_reg);
key_all = make_call_key(head_reg, tail_reg);


bucket_tail = read_symbtab_bucket(Call_Cache, key_tail);
bucket_all = read_symbtab_bucket(Call_Cache, key_all);


if ( (bucket_tail != NULL)  &&  (bucket_all == NULL) )
/* new insertion  */
 {
   store_in_cache(key_all, 1);
   *update_found = 1;
 }
else /* no new insertion */
 {
    free(key_all);
 }

free(key_tail);
    

}  /* update_step    */







int update_compose(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
int update_found = 0;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupdate_compose:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif






/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);

 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\nupdate_compose:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif

       /*   prg_reg != NULL */

update_step(
	p, 
	calling_reg, 
	regfield(calling_reg, compose_word0), 
	prg_reg, 
	&update_found);


update_step(
	p, 
	calling_reg, 
	regfield(calling_reg, compose_word0 + 2), 
	prg_reg, 
	&update_found);



prg_reg = regfield(prg_reg, ptr_down);


  }  /*  while  */




#if (dbg_flag > 100)
printf("\nupdate_compose:%u: end: return %d", count, update_found);
#endif

return ( update_found );

}   /* update_compose */









int update_prj(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
int update_found = 0;

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupdate_prj:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);

 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\nupdate_prj:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif

       /*   prg_reg != NULL */

update_step(
	p, 
	calling_reg, 
	regfield(calling_reg, prj_word0), 
	prg_reg, 
	&update_found);





prg_reg = regfield(prg_reg, ptr_down);

  }   /*  while  */



#if (dbg_flag > 100)
printf("\nupdate_prj:%u: end: return %d", count, update_found);
#endif


return ( update_found );

}   /* update_ifte  */














int update_join(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
int update_found = 0;

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupdate_join:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);

 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\nupdate_join:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif

       /*   prg_reg != NULL */

for (
i = 0; 
i < (unsigned int) regfield(calling_reg, ordlist_word0);
i++)
   {

update_step(
	p, 
	calling_reg, 
	vtxeval(regfield(calling_reg, ordlist_word0 + 1), i), 
	prg_reg, 
	&update_found);

   }  /*  for i */




prg_reg = regfield(prg_reg, ptr_down);

  }   /*  while  */



#if (dbg_flag > 100)
printf("\nupdate_join:%u: end: return %d", count, update_found);
#endif


return ( update_found );

}   /* update_join  */





void def1step_calls(prghdr* p)
{
/* dcl */
void* prg_reg;
void* prg_vtx;


#if (dbg1 > 0)
static int count = 0;
unsigned int ii = 0;
count++;
#endif


/* prg  */

#if (dbg1 > 100)
printf("\ndef1step_calls:%u: begin: program = 0x%x", count, p);
#endif



/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);


 while (prg_reg != NULL)
  {

#if (dbg1 > 100)
ii++;
printf("\ndef1step_calls:%u: begin while prg_reg = 0x%x: loop %u", 
count, 
prg_reg, 
ii
);
#endif

       /*   prg_reg != NULL */

       prg_vtx = regeval(prg_reg);

/* note that are no xarg, boole instr here */

       switch( (int) vtxeval(prg_vtx, op_code) )
          {


           case tk_ifte:

                    ifte1step(p, prg_reg);

#if (dbg1 > 100)
printf("\ndef1step_calls:%u: ifte %u", count, ii);
#endif


                     break;



           case tk_compose:
 
                     compose1step(p, prg_reg);

#if (dbg1 > 100)
printf("\ndef1step_calls:%u: compose %u", count, ii);
#endif

                     break;



           case tk_prj:
 
                     prj1step(p, prg_reg);

#if (dbg1 > 100)
printf("\ndef1step_calls:%u: prj %u", count, ii);
#endif

                     break;





           case tk_join:
 
                     join1step(p, prg_reg);

#if (dbg1 > 100)
printf("\ndef1step_calls:%u: join %u", count, ii);
#endif

                     break;




           default:  /* do nothing on query, xarg, boole */
          
                     break;

          }




       prg_reg = vtxeval(prg_vtx, ptr_down);


  }   /* while */


#if (dbg1 > 100)
printf("\ndef1step_calls:%u: end: program = 0x%x", count, p);
#endif


}   /* def1step_calls  */









void ifte1step(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
unsigned int i;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nifte1step:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



for (i = 0; i < 3; i++)
   {

update1step(
	p, 
	calling_reg, 
	regfield(calling_reg, ifte_word0 + i)
           );

   }  /*  for i */



#if (dbg_flag > 100)
printf("\nifte1step:%u: end", count);
#endif

}   /* update_ifte  */








void update1step(
	prghdr* p, 
	void* lhs_reg, 
	void* rhs_reg
)
{
/*  dcl */
char *key;
void *bucket;

/* prg */

key = make_call_key(lhs_reg, rhs_reg);

bucket = read_symbtab_bucket(Call_Cache, key);


if ( bucket == NULL )
/* insert  */
 {
   store_in_cache(key, 1);
 }

    

}  /* update1step    */







void compose1step(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
int update_found = 0;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\ncompose1step:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



update1step(
	p, 
	calling_reg, 
	regfield(calling_reg, compose_word0)
           );


update1step(
	p, 
	calling_reg, 
	regfield(calling_reg, compose_word0 + 2)
           );




#if (dbg_flag > 100)
printf("\ncompose1step:%u: end", count);
#endif


}   /* update_compose */







void prj1step(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
unsigned int i;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nprj1step:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



update1step(
	p, 
	calling_reg, 
	regfield(calling_reg, prj_word0)
           );




#if (dbg_flag > 100)
printf("\nprj1step:%u: end", count);
#endif

}   /* update_prj  */









void join1step(prghdr* p, void* calling_reg)
{
/* dcl */
void *prg_reg;
unsigned int i;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\njoin1step:%u: begin: program = 0x%x. calling_reg = 0x%x", 
count, p, calling_reg
);
#endif



for (
i = 0; 
i < (unsigned int) regfield(prg_reg, ordlist_word0); 
i++)
   {

update1step(
	p, 
	calling_reg, 
	vtxeval(regfield(calling_reg, ordlist_word0 + 1), i)
           );

   }  /*  for i */



#if (dbg_flag > 100)
printf("\njoin1step:%u: end", count);
#endif

}   /* update_join   */







/*
  Returns true iff calling_reg calls called_reg 
  in program p.
  HYP: calling_reg is boole, xarg, ifte, compose.
*/











int reg_calls(prghdr *p, void *calling_reg, void *called_reg)
{
/* dcl */
char *key;
void *bucket;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nreg_calls:%u: begin: prg = 0x%x, calling = 0x%x, called = 0x%x", 
count, p, calling_reg, called_reg);
#endif


/*  A defined register does not call an undefined register */
if ( (regfield(calling_reg, bddvalue) != NULL)
     &&
     (regfield(called_reg, bddvalue) == NULL)
   )
/* no calls */
{
#if (dbg_flag > 100)
printf("\nreg_calls:%u: calling_reg defined, called_reg undefined: return 0", 
count);
#endif

   return (0);
}


#if (dbg_flag > 100)
printf("\nreg_calls:%u: check xarg, boole", 
count);
#endif

/*  xarg and boole do not call any register  */

if ( ( (int) vtxeval(regeval(calling_reg), op_code) == tk_xarg)
     || 
     ( (int) vtxeval(regeval(calling_reg), op_code) == tk_boolean)
   )
/* no calls */
{
#if (dbg_flag > 100)
printf("\nreg_calls:%u: calling_reg is xarg or boole: return 0", 
count);
#endif

   return (0);
}



#if (dbg_flag > 100)
printf("\nreg_calls:%u: check cache 0x%x", 
count, Call_Cache);
#endif

/* Check if result is in Cache  */
key = make_call_key(calling_reg, called_reg);


#if (dbg_flag > 100)
printf("\nreg_calls:%u: key = %s", 
count, key);
#endif

bucket = read_symbtab_bucket(Call_Cache, key);


if (bucket != NULL)
/* result in cache */
{

#if (dbg_flag > 100)
printf("\nreg_calls:%u: edge in cache: key = %s, return 1", 
count, 
key
);
#endif

    return ( 1 );

}

#if (dbg_flag > 100)
printf("\nreg_calls:%u: edge not cache: return 0", 
count);
#endif


return (0);



}    /*  reg_calls  */





void store_in_cache(char *key, unsigned int value)
{
  /*  dcl  */
void *bucket;

  /*  prg  */
bucket = get_new_stbk();
write_stbk_key(bucket, key);
write_stbk_tkcode(bucket, tk_id);
write_stbk_info(bucket, (void *) value);
write_symbtab_bucket(Call_Cache, bucket);


}   /*  store_in_cache    */





char* make_call_key(void *calling_reg, void *called_reg)
{
/*  dcl */
char *p;


/*  prg */

p = (char *) malloc(sizeof(char) * KEY_SIZE);

if (p == NULL)
/* no space */
{
    printf("\nmake_call_key: error: no malloc space\n");
    exit (1);
}

/* ok */

sprintf(p, "p%x.%x", calling_reg, called_reg);


return (p);



}   /*  make_call_key    */
