/*
*************************************
file: free.c

Define free.


Free reset reg to 0.

Hyp: eq.adr in bddvalue
(defined during loopback definition).

p -> pc contains loop-back_region,
i.e. first instruction target of a loopback
containing more than one instruction.
p -> pc is initialized to NULL.
*************************************
*/

/*
*******************************
When supporting exec
do not free xarg and boole.
*******************************
*/


#include "local.h"
#include <limits.h>


#define dbg_flag 0   /* normal value: 0  */


/*
if no_free_xb == 0 then free on xarg and boole
if no_free_xb != 0 then no free for xarg or boole.
Note: to support exec we need no_free_xb != 0
*/

#define no_free_xb 0 


static void* present_loopback;
static void* previous_loopback;


static void* get_free_reg();

static void* insert_free(prghdr *p, void *prg_reg);

static void free_at_end(void *prg_reg, void *end_of_prg);

static void reset_free(prghdr *p);

static void compose_free(prghdr *p, void *prg_reg);

static void ifte_free(prghdr *p, void *prg_reg);

static void prj_free(prghdr *p, void *prg_reg);

static void join_free(prghdr *p, void *prg_reg);

static void lhs_rhs_free(prghdr *p, void *lhs_reg, void *rhs_reg);

static void upd_lpbk_region(prghdr *p, void *prg_reg);

static void init_mark(prghdr *p);

static int is_ancestor(void *prg_reg, void *target_reg);

static void mark_query_reg(prghdr *p);




void define_free(prghdr* p)
{
/* dcl */
void* prg_reg;
void *end_of_prg;  /* end instruction in prg */

int update_found;

unsigned int adr_loopback_region = UINT_MAX;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\ndefine_free:%u: begin: program = 0x%x", count, p);
#endif

#if (no_free_xb != 0)
printf("\nbp20: no free done on xarg and boole to support exec");
#endif

#if (no_free_xb == 0)
printf("\nbp20: free done on xarg and boole: exec not supported");
#endif


reset_free(p);

init_mark(p);


mark_query_reg(p);


present_loopback = NULL;

previous_loopback = NULL;


#if (dbg_flag > 100)
printf("\ndefine_free:%u: loopback, ord: initialized", count);
#endif



/* initialize p -> pc */
p -> pc = NULL;



/*  first instruction  */
prg_reg = p -> prg;


 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\ndefine_free:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif

/* update end_of_prg */
      end_of_prg = prg_reg;


       switch( (int) regfield(prg_reg, op_code) )
          {


           case tk_ifte:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: ifte: lhs_reg = 0x%x", count, prg_reg);
#endif

if ( regfield(prg_reg, mark0) != NULL )
  /* upd  */
  {
     regfield(prg_reg, mark1) = present_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.1: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


     previous_loopback = present_loopback;
     present_loopback = regfield(prg_reg, mark0);
  }
else
  {
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.2: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif

  
  }


#if (dbg_flag > 100)
printf("\ndefine_free:%u: ifte: prs_lb=0x%x, prv_lb=0x%x", count, 
present_loopback,
previous_loopback
);
#endif
  




                    ifte_free(p, prg_reg);



                     break;




           case tk_compose:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: compose: lhs_reg = 0x%x", count, prg_reg);
#endif

if ( regfield(prg_reg, mark0) != NULL )
  /* upd  */
  {
     regfield(prg_reg, mark1) = present_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.3: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif

     previous_loopback = present_loopback;
     present_loopback = regfield(prg_reg, mark0);
  }
else
  {
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.4: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif

  
  }

#if (dbg_flag > 100)
printf("\ndefine_free:%u: compose: prs_lb=0x%x, prv_lb=0x%x", count, 
present_loopback,
previous_loopback
);
#endif

 
                   compose_free(p, prg_reg);
                

                   break;




           case tk_join:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: join: lhs_reg = 0x%x", count, prg_reg);
#endif

if ( regfield(prg_reg, mark0) != NULL )
  /* upd  */
  {
     regfield(prg_reg, mark1) = present_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.5: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


     previous_loopback = present_loopback;
     present_loopback = regfield(prg_reg, mark0);
  }
else
  {
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.6: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


  
  }

#if (dbg_flag > 100)
printf("\ndefine_free:%u: join: prs_lb=0x%x, prv_lb=0x%x", count, 
present_loopback,
previous_loopback
);
#endif


 
                   join_free(p, prg_reg);
                

                   break;




           case tk_prj:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: prj: lhs_reg = 0x%x", count, prg_reg);
#endif

if ( regfield(prg_reg, mark0) != NULL )
  /* upd  */
  {
     regfield(prg_reg, mark1) = present_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.7: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


     previous_loopback = present_loopback;
     present_loopback = regfield(prg_reg, mark0);
  }
else
  {
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.8: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif

  
  }



#if (dbg_flag > 100)
printf("\ndefine_free:%u: prj: prs_lb=0x%x, prv_lb=0x%x", count, 
present_loopback,
previous_loopback
);
#endif

 
                   prj_free(p, prg_reg);
                

                   break;





           case tk_loopback:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: loopback: lhs_reg = 0x%x", count, prg_reg);
#endif


 
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free:: ck mark1.9: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif



    present_loopback = previous_loopback;
    previous_loopback = regfield(regfield(prg_reg, loopback_word0), mark1);

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.10: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif



#if (dbg_flag > 100)
printf("\ndefine_free:%u: loopback: prs_lb=0x%x, prv_lb=0x%x", count, 
present_loopback,
previous_loopback
);
#endif


#if 0

                   upd_lpbk_region(p, prg_reg);
#endif


#if (dbg_flag > 100)
printf("\ndefine_free:%u: after upd_lpbk_region: p -> pc = 0x%x", count, p -> pc);
#endif
                

                   break;



           case tk_exec:
           case tk_xarg:
           case tk_boolean:

#if (dbg_flag > 100)
printf("\ndefine_free:%u: exec, xarg, boolean: lhs_reg = 0x%x", count, prg_reg);
#endif

if ( regfield(prg_reg, mark0) != NULL )
  /* upd  */
  {
     regfield(prg_reg, mark1) = present_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.11: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


     previous_loopback = present_loopback;
     present_loopback = regfield(prg_reg, mark0);
  }
else
  {
    regfield(prg_reg, mark0) = present_loopback;
    regfield(prg_reg, mark1) = previous_loopback;

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.12: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif

  
  }


regfield(prg_reg, loopback) = prg_reg;

                

                   break;


#if 0
/* no query in prg list */

  case tk_isone:
  case tk_eval:
  case tk_satasg:
  case tk_C:


#if 0
fprintf(stderr, "\nfree: query 0x%x checked, rhs: 0x%x", 
prg_reg, 
regfield(prg_reg, query_word0)
);
#endif


regfield(regfield(prg_reg, query_word0), loopback) = prg_reg;

                   
                     break;

#endif




           default:  /* do nothing on xarg, boole */

/* Note: no regs on rhs of xarg or boole  */
          
                     break;

          }




       prg_reg = regfield(prg_reg, ptr_down);


  }   /* while */




/*  prg_reg == NULL  */
/*  end_of_prg != NULL, 
    regfield(end_of_prg, op_code) == tk_end 
*/

/* 
   if p -> pc == NULL there is no loopback region.
*/

/* define adress loopback region */
/* init: adr_loopback_region == UINT_MAX; (no loppback region) */



#if (dbg_flag > 100)
printf("\ndefine_free:%u: insert free's, p -> pc = 0x%x, adr_loopback_region = %u", 
count, p -> pc, adr_loopback_region);
#endif


/*  insert free's  */


/*  first instruction  */
prg_reg = p -> prg;


 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\ndefine_free:%u: begin insert while: lhs_reg = 0x%x", 
count, prg_reg);
#endif


       switch( (int) regfield(prg_reg, op_code) )
          {


           case tk_ifte:
           case tk_compose:
           case tk_prj:
           case tk_join:
           case tk_exec:

#if (no_free_xb == 0)
           case tk_xarg:
           case tk_boolean:
#endif



#if (dbg_flag > 100)
printf("\ndefine_free:%u: ifte/compose/xarg/boole/prj/join", count);
#endif


/*  loopback != NULL  */

#if (dbg_flag > 100)
printf("\ndefine_free:%u: begin insert free: prg_reg = 0x%x", 
count, 
prg_reg
);
#endif

                          insert_free(p, prg_reg);

#if (dbg_flag > 100)
printf("\ndefine_free:%u: end insert free: prg_reg = 0x%x", 
count, 
prg_reg
);
#endif


                     break;



           default:  /* do nothing on query  */

/* Note: no regs on rhs of xarg or boole  */
          
                     break;

          }




       prg_reg = regfield(prg_reg, ptr_down);


  }   /* while */







#if (dbg_flag > 100)
printf("\ndefine_free:%u: end: program = 0x%x", count, p);
#endif


}   /* prg  */












void lhs_rhs_free(prghdr *p, void *lhs_reg, void *rhs_reg)
{
/* dcl */
unsigned int rhs_stratum;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nlhs_rhs_free:%u: begin: prg 0x%x, lhs_reg 0x%x, rhs_reg 0x%x, rhs.loopback = 0x%x", 
count, 
p, lhs_reg, rhs_reg,
regfield(rhs_reg, loopback)
);
#endif


regfield(rhs_reg, loopback) = lhs_reg;



#if (dbg_flag > 100)
printf("\nlhs_rhs_free:%u: loopback of rhs 0x%x = 0x%x", 
count, 
rhs_reg, 
regfield(rhs_reg, loopback)
);
#endif


}  /*  lhs_rhs_free   */







void ifte_free(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

lhs_rhs_free(p, prg_reg, regfield(prg_reg, ifte_word0));

lhs_rhs_free(p, prg_reg, regfield(prg_reg, ifte_word0 + 1));

lhs_rhs_free(p, prg_reg, regfield(prg_reg, ifte_word0 + 2));


}  /*  ifte_free   */


   




void compose_free(prghdr *p, void *prg_reg)
{
/* dcl */



/* prg */

/* check rhs word 0, 2 */
lhs_rhs_free(p, prg_reg, regfield(prg_reg, compose_word0));

lhs_rhs_free(p, prg_reg, regfield(prg_reg, compose_word0 + 2));



}  /*  compose_free  */








void prj_free(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

lhs_rhs_free(p, prg_reg, regfield(prg_reg, prj_word0));

}  /*  ifte_free   */





void join_free(prghdr *p, void *prg_reg)
{
/* dcl */
unsigned int i;


/* prg */

      
                    for (
i = 0; 
i < (unsigned int) regfield(prg_reg, ordlist_word0);
i++
)
                       {

lhs_rhs_free(
p, 
prg_reg, 
vtxeval(regfield(prg_reg, ordlist_word0 + 1), i)
);


                       } /* for */


}  /*  join_free   */








/* set loopback to reg value */

void reset_free(prghdr *p)
{
/* dcl */
void *prg_reg;
unsigned int i = 0;

/* prg */


prg_reg = p -> prg;

 while (prg_reg != NULL)
  {
 
       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

           case tk_xarg:
           case tk_boolean:
           case tk_ifte:
           case tk_compose:
           case tk_join:
           case tk_prj:
           case tk_exec:
           case tk_loopback:

                        regfield(prg_reg, loopback) = prg_reg;
                        regfield(prg_reg, bddvalue) = NULL;

                        regfield(prg_reg, mark0) = NULL;
                        regfield(prg_reg, mark1) = NULL;

#if (dbg_flag > 100)
printf("\ndefine_free: ck mark1.12: regfield(0x%x, 0x%x) = 0x%x", 
prg_reg,
mark1,
regfield(prg_reg, mark1)
);
#endif


                        regfield(prg_reg, mark2) = NULL;


                        break;




           default:  /* do nothing on query and paddings */
          
                     break;

          }




       prg_reg = vtxeval(regeval(prg_reg), ptr_down);


  }   /* while */





}   /*  reset_free  */













/* initialize mark0 */

void init_mark(prghdr *p)
{
/* dcl */
void *prg_reg;
unsigned int i = 0;

/* prg */


prg_reg = p -> prg;

 while (prg_reg != NULL)
  {
 
       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

            case tk_loopback:


  regfield(regfield(prg_reg, loopback_word0), mark0) = 
  regfield(prg_reg, loopback_word0);

  regfield(prg_reg, mark0) = regfield(prg_reg, loopback_word0);


/*   mark headers  with pointer to (nonfree) 
     instruction before loopback 
*/

  regfield(regfield(prg_reg, loopback_word0), bddvalue) = 
  regfield(prg_reg, ptr_up);

 

#if (dbg_flag > 100)
printf("\ninit_mark: loopback: prg_reg = 0x%x, regfield(prg_reg, mark0) = 0x%x", 
prg_reg,
regfield(prg_reg, mark0)
);
printf("\nregfield(regfield(prg_reg, mark0), mark0) = 0x%x", 
regfield(regfield(prg_reg, mark0), mark0)
);
#endif


                        break;



           default:  /* do nothing on query and paddings */
          
                     break;

          }




       prg_reg = vtxeval(regeval(prg_reg), ptr_down);


  }   /* while */





}   /*  init_mark0  */











/* mark regs on witch there are queries  */

void mark_query_reg(prghdr *p)
{
/* dcl */
void *query_reg;

/* prg */

query_reg = p -> querylist;


 while (query_reg != NULL)
  {


       regfield(regfield(query_reg, query_word0), mark2) = (void *) p;


/* move to next query */
       query_reg = regfield(query_reg, ptr_down);


  }   /* while */



}   /*  mark_query_reg()  */











/* insert free instruction for prg_reg
   return pointer to new free inserted

   Insertion ASAP after last rhs occurrence
   not ancestor loopback region of lhs
   */


/*
****************
Note: 2 frees needed:
first reset bdd value,
second one delete definition.
****************
*/

void* insert_free(prghdr *p, void *prg_reg)
{
/* dcl */
void *new_reg, *new_reg2;
void *target_reg;

unsigned int free_inserted = 0;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\ninsert_free:%u: begin: prg_reg = 0x%x", 
count, prg_reg
);
#endif


new_reg = get_free_reg();
regfield(new_reg, free_word0) = prg_reg;

new_reg2 = get_free_reg();
regfield(new_reg2, free_word0) = prg_reg;



/*  stack new_reg 
    after
    regfield(prg_reg, loopback) 
    and not in ancestor loopback region
    and not in header of same loopback region.
*/


target_reg = regfield(regfield(prg_reg, loopback), ptr_down);

while (target_reg != NULL)
    {

#if (dbg_flag > 100)
printf("\ninsert_free:%u: begin while: target_reg = 0x%x", 
count, 
target_reg
);
#endif



       switch( (int) regfield(target_reg, op_code) )
          {

           case tk_ifte:
           case tk_compose:
           case tk_prj:
           case tk_join:
           case tk_exec:


        
           case tk_xarg:
           case tk_boolean:
           case tk_loopback:


 
 if ( (regfield(target_reg, mark0) != NULL) /* in loopback region */
      &&
      (regfield(prg_reg, mark2) == (void *) p) /* prg_reg used in query */
    )
/* no free in loopback region for regs used in queries */
   {
        break;

   }



/* prg_reg is not used in a query */



     if (   (regfield(prg_reg, mark0) != NULL) /* in loopback region */
            && 
            (regfield(prg_reg, mark0) == regfield(target_reg, mark0))
            && 
            (regfield(prg_reg, bddvalue) != NULL)  /* header loopback regon */
            &&
            (free_inserted != 0)
        )
            /* prg_reg header in same region of target, no second free */
            {

                break;

            }





/* No free if prg_reg and target in same region
   and prg_reg is nonfree instruction
   before loopback.
   In fact in this case we need to store
   prg_reg value across iterations.
*/

    if ( regfield(target_reg, mark0) != NULL )
       {

        if ( (regfield(prg_reg, mark0) == regfield(target_reg, mark0))
             && 
   ( regfield(regfield(prg_reg, mark0), bddvalue)
     == prg_reg
   )
           )
            /* prg_reg followed by loopback, no free */
            {

                break;

            }
       }



if ( is_ancestor(prg_reg, target_reg) )
   /* ancestor: free is not possible */
   { 

#if (dbg_flag > 100)
printf("\ninsert_free:%u: then: target_reg=0x%x  ancestor", 
count, 
target_reg
);
#endif

      break;
   }




/* ok: not ancestor loopback region */

if (free_inserted == 0)
     {  /* first free */

                           /*   insert (stack) free  */

stack_newreg(new_reg, target_reg);

/* record free adr in loopback for use 
   in query scheduling
*/

regfield(prg_reg, loopback) = new_reg;


free_inserted++;


       
     }  /* first free */








if (free_inserted == 1)
     {  /* second free */

/*
   only in no loopback region 
*/

  if ( regfield(target_reg, mark0) == NULL )
 /* target, not in loopback region */
         {


                  /*   insert (stack) free  */

stack_newreg(new_reg2, target_reg);

free_inserted++;


                        return (new_reg2);

         }   /* target, not in loopback region */

       
     }  /* second free */




                      break;





           case tk_end:



if (free_inserted == 0)
     {  /* first free */

                           /*   insert (stack) free  */

stack_newreg(new_reg, target_reg);

/* record free adr in loopback for use 
   in query scheduling
*/

regfield(prg_reg, loopback) = new_reg;


free_inserted++;


stack_newreg(new_reg2, target_reg);


free_inserted++;


                        return (new_reg2);
       
     }  /* first free */





if (free_inserted == 1)
     {  /* second free */

 
                  /*   insert (stack) free  */

stack_newreg(new_reg2, target_reg);



free_inserted++;


                        return (new_reg2);

       
     }  /* second free */








           default:  /* not found */
          
                    break;

          }  /* switch */




    target_reg = regfield(target_reg, ptr_down);


    } /* while */






#if (dbg_flag > 100)
printf("\ninsert_free:%u: end: prg_reg = 0x%x", 
count, prg_reg
);
#endif

}  /*  append_loopback  */







/*
Insert free at end for prg_reg
*/




void free_at_end(void *prg_reg, void *end_of_prg)
{
/* dcl */
void *new_reg;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nfree_at_end:%u: begin: prg_reg = 0x%x", 
count, prg_reg
);
#endif


new_reg = get_free_reg();

regfield(new_reg, free_word0) = prg_reg;

stack_newreg(new_reg, end_of_prg);


/* record free adr in loopback for use 
   in query scheduling
*/

regfield(prg_reg, loopback) = new_reg;


#if (dbg_flag > 100)
printf("\nfree_at_end:%u: end: prg_reg = 0x%x", 
count, prg_reg
);
#endif

}  /*  append_loopback  */






void* get_free_reg()
{
/* dcl */
void *new_reg;


/* prg */

new_reg = get_prgvtx(1);
regeval(new_reg) = get_prgvtx(size_free);
regfield(new_reg, op_code) = (void *) tk_free;

return (new_reg);

}







#if 0

/* hyp: regfield(prg_reg, op_code) == tk_loopback */

void upd_lpbk_region(prghdr *p, void *prg_reg)
{
/*  dcl  */



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: begin: prg_reg = 0x%x, loopback = 0x%x, ptr_up = 0x%x", 
count, prg_reg, 
regfield(prg_reg, loopback_word0), 
regfield(prg_reg, ptr_up) 
);
#endif







/*  check if loopback includes more than 1 reg */
if ( regfield(prg_reg, loopback_word0)
     ==
     regfield(prg_reg, ptr_up)
   )
/* just 1 reg */
   {

#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: just 1 reg in loopback", count);
#endif

      return ; 
   }


#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: more thatn 1 reg in loopback", count);
#endif

/*  more than 1 reg loopback */


if (p -> pc == NULL)
/* no previous value */
  {

#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: pc == NULL, update pc", count);
#endif

     p -> pc = regfield(prg_reg, loopback_word0);

#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: pc = 0x%x", count, p -> pc);
#endif

     return;
  }



#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: pc = 0x%x != NULL", count, p -> pc);
#endif


/* p -> pc != NULL */


if ( regfield(regfield(prg_reg, loopback_word0), bddvalue)
     <
     regfield(p -> pc, bddvalue)
   )
/* update */
  {
      p -> pc = regfield(prg_reg, loopback_word0);


#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: new value: pc = 0x%x", count, p -> pc);
#endif


      return;

  }


#if (dbg_flag > 100)
printf("\nupd_lpbk_region:%u: pc = 0x%x: no updating", count, p -> pc);
#endif

}   /* upd_lpbk_region   */


#endif







int is_ancestor(void *prg_reg, void *target_reg)
{

#if (dbg_flag > 100)
printf("\nis_ancestor:0x%x,0x%x: begin: opcodes: '%c'=%u, '%c'=%u", 
prg_reg,
target_reg,
regfield(prg_reg, op_code),
regfield(prg_reg, op_code),
regfield(target_reg, op_code),
regfield(target_reg, op_code)
);
#endif


  if (regfield(target_reg, mark0) == NULL)
    /* not ancestor */
    {

#if (dbg_flag > 100)
printf("\nis_ancestor:0x%x,0x%x: NULL prs_lb: not ancestor: return 0",
prg_reg,
target_reg
);
#endif

       return (0);

    }


/* regfield(target_reg, mark0) != NULL  */




  if ( regfield(prg_reg, mark0) == regfield(target_reg, mark0) )
     /* not ancestor */
    {

#if (dbg_flag > 100)
printf("\nis_ancestor:0x%x,0x%x: same region: not ancestor: return 0",
prg_reg,
target_reg
);
#endif

        return (0);
    }


/* !=  */

  if ( regfield(prg_reg, mark0) == regfield(target_reg, mark1) )
     /* ancestor */
    {

#if (dbg_flag > 100)
printf("\nis_ancestor:0x%x,0x%x: ancestor: return 1",
prg_reg,
target_reg
);
#endif

        return (1);
    }
  else 
    {
#if (dbg_flag > 100)
printf("\nis_ancestor:0x%x 0x%x 0x%x 0x%x: recursion:",
prg_reg,
target_reg,
mark1,
regfield(target_reg, mark1)
);
#endif

       return ( is_ancestor(prg_reg, regfield(target_reg, mark1))  );
    }

}  /* is_ancestor  */
