/*
*************************************
file: loopopt.c

Esy part of loop optimization.



*************************************
*/


#include "local.h"



#define dbg_flag 0 /* normal value: 0  */



 
static 
int update_reg_stratum(prghdr *p, void *lhs_reg, void *rhs_reg);


static
int update_ifte_stratum(prghdr *p, void *prg_reg);

static
int update_compose_stratum(prghdr *p, void *prg_reg);

static
int update_join_stratum(prghdr *p, void *prg_reg);

static
int update_prj_stratum(prghdr *p, void *prg_reg);



static
void reset_stratum(prghdr *p);

static
void order_stratums(prghdr *p);





void loop_optmz2(prghdr* p)
{
/* dcl */
void* prg_reg;
void* prg_vtx;

int move_done;


#if (dbg_flag > 0)
static int count = 0;
unsigned int ii1 = 0;
unsigned int ii2 = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: begin: program = 0x%x", count, p);
#endif


/*  Start call env.  */
start_call_env();


#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: call_env defined", count);
#endif


/* define calls */
define_calls(p);


#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: calls defined", count);
#endif



/* pc initialized in loop1 p -> pc != NULL  */


/* initialize level values (from pc) in loopback  */
reset_stratum(p);

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: stratums resetted", count);
#endif


move_done = 1;

while (move_done != 0)
 {

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: begin while move_done: ii1 = %u", count, ii1);
#endif

move_done = 0;

/*  first instruction not inserted on top */
prg_reg = regfield(p -> pc, ptr_down);


 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: begin while prg_reg = 0x%x, ii2 = %u", 
count, prg_reg,
ii2
);
#endif

       /*   prg_reg != NULL */

       prg_vtx = regeval(prg_reg);

/* note that there are no xarg, boole instr here */

       switch( (int) vtxeval(prg_vtx, op_code) )
          {


           case tk_ifte:

                     move_done = 
move_done || update_ifte_stratum(p, prg_reg);

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: ifte: move_done = %d", count, move_done);
#endif


                     break;



           case tk_compose:
 
                     move_done = 
move_done || update_compose_stratum(p, prg_reg);

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: compose: move_done = %d", count, move_done);
#endif

                     break;






           case tk_prj:
 
                     move_done = 
move_done || update_prj_stratum(p, prg_reg);

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: prj: move_done = %d", count, move_done);
#endif

                     break;






           case tk_join:
 
                     move_done = 
move_done || update_join_stratum(p, prg_reg);

#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: join: move_done = %d", count, move_done);
#endif

                     break;





           default:  /* do nothing on query, xarg, boole */
          
                     break;

          }




       prg_reg = vtxeval(prg_vtx, ptr_down);


  }   /* while */


}  /*  while */



/*  end call env.  */
end_call_env();


#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: order stratums", count);
#endif

/* order instructions (after pc) according to stratum */

order_stratums(p);


#if (dbg_flag > 100)
printf("\nloop_optmz2:%u: end: program = 0x%x", count, p);
#endif


}   /* prg  */















/*
   HYP:  vtxeval(regeval(prg_reg), op_code) == tk_ifte

   Returns true (nonzero) iff prg_reg 
   new stratum is different from old one.

*/


int update_reg_stratum(prghdr *p, void *lhs_reg, void *rhs_reg)
{
/* dcl */
unsigned int rhs_stratum;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: begin: prg 0x%x, lhs_reg 0x%x, rhs_reg 0x%x", 
count, p, lhs_reg, rhs_reg
);
#endif

if (
      reg_calls(p, rhs_reg, lhs_reg)
   )
/* loop */
   {

#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: loop: rhs_stratum = %u", 
count, regfield(rhs_reg, loopback)
);
#endif

        rhs_stratum = (unsigned int) regfield(rhs_reg, loopback);
   }
else  /* no loop */
   {
#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: no loop: rhs_stratum = %u", 
count, 
1 + ((unsigned int) regfield(rhs_reg, loopback))
);
#endif

       rhs_stratum = 1 + ((unsigned int) regfield(rhs_reg, loopback));
   }



#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: lhs_stratum = %u", 
count, 
regfield(lhs_reg, loopback)
);
#endif



if ( (unsigned int) regfield(lhs_reg, loopback)
     <
     rhs_stratum
   )
/*  updating */
{

#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: updating: new lhs_stratum = %u; return 1", 
count, 
rhs_stratum
);
#endif

 regfield(lhs_reg, loopback) = (void *) rhs_stratum;
 
 return (1); 

}
else
/*  no updating */
{ 
#if (dbg_flag > 100)
printf("\nupdate_reg_stratum:%u: no updating; return 0", 
count, 
rhs_stratum
);
#endif

 return (0); 
}



}   /*  update_reg_stratum  */







int update_ifte_stratum(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

return (
           update_reg_stratum(p, prg_reg, regfield(prg_reg, ifte_word0))
        ||
           update_reg_stratum(p, prg_reg, regfield(prg_reg, ifte_word0 + 1))
        ||
           update_reg_stratum(p, prg_reg, regfield(prg_reg, ifte_word0 + 2))
       );

}  /*  update_ifte_stratum   */


   





/*
   HYP:  vtxeval(regeval(prg_reg), op_code) == tk_compose

   Returns true (nonzero) iff prg_reg can be moved
   (i.e. its rhs symbols are defined).

*/


int update_compose_stratum(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */


return (
           update_reg_stratum(p, prg_reg, regfield(prg_reg, compose_word0))
        ||
           update_reg_stratum(p, prg_reg, regfield(prg_reg, compose_word0 + 2))
       );




}  /*  update_compose_stratum  */









int update_prj_stratum(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

return (
           update_reg_stratum(p, prg_reg, regfield(prg_reg, prj_word0))
       );

}  /*  update_prj_stratum   */






int update_join_stratum(prghdr *p, void *prg_reg)
{
/* dcl */
unsigned int i;
int result = 0;

/* prg */


           
                    for (
i = 0; 
i < (unsigned int) regfield(prg_reg, ordlist_word0);
i++
)
                       {


result = result || 
update_reg_stratum(
p, 
prg_reg,
vtxeval(regfield(prg_reg, ordlist_word0 + 1), i)
);



                       } /* for */



return ( result );

}  /*  update_prj_stratum   */










/* set strata to 0 */

void reset_stratum(prghdr *p)
{
/* dcl */
void *prg_reg;

/* prg */


#if 0
/*  first instruction after pc */
prg_reg = regfield(p -> pc, ptr_down);
#endif

/* just reset all regs  */
prg_reg = p -> prg;





 while (prg_reg != NULL)
  {
 
       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

           case tk_xarg:
           case tk_boolean:
           case tk_ifte:
           case tk_compose:
           case tk_join:
           case tk_prj:


                        regfield(prg_reg, loopback) = (void *) 0;


                        break;



           default:  /* do nothing on query and paddings */
          
                     break;

          }




       prg_reg = vtxeval(regeval(prg_reg), ptr_down);


  }   /* while */





}   /*  reset_stratum  */














/* order stratums */

void order_stratums(prghdr *p)
{
/* dcl */
void *prg_reg, *next_reg;
void *last_inserted;

unsigned int stratum_count = 0;

int insertion_done;





#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\norder_stratums:%u: begin: prg 0x%x, p -> pc = 0x%x", 
count, p, p -> pc
);
#endif



last_inserted = p -> pc;

insertion_done = 1;


while ( insertion_done )

{

#if (dbg_flag > 100)
printf("\norder_stratums:%u: while insertion_done: last_inserted = 0x%x", 
count, last_inserted
);
#endif

  insertion_done = 0;

prg_reg = regfield(last_inserted, ptr_down);

 while (prg_reg != NULL)
  {
 
#if (dbg_flag > 100)
printf("\norder_stratums:%u: while prg_reg: prg_reg = 0x%x: stratum = %u", 
count, prg_reg, stratum_count
);
#endif

next_reg = regfield(prg_reg, ptr_down);

#if (dbg_flag > 100)
printf("\norder_stratums:%u: next_reg = 0x%x", 
count, next_reg
);
#endif

       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

           case tk_ifte:
           case tk_compose:
           case tk_join:
           case tk_prj:


#if (dbg_flag > 100)
printf("\norder_stratums:%u: ifte/compose: prg_reg.loopback = %u, stratum = %u", 
count, 
regfield(prg_reg, loopback),
stratum_count
);
#endif


        insertion_done = 1;

                        if ( regfield(prg_reg, loopback) 
                             ==
                             (void *) stratum_count
                           )
            {
         last_inserted = move_reg(p, last_inserted, prg_reg);

#if (dbg_flag > 100)
printf("\norder_stratums:%u: called move_reg: last_inserted = 0x%x", 
count, 
last_inserted
);
#endif

            }



                        break;



           default:  /* do nothing on query and paddings */
          
                     break;

          }   /*  switch */




       prg_reg = next_reg;


  }   /* while */


/* next stratum  */
stratum_count++;

}   /*  while insertion_done  */





}   /*  order_stratums  */






