/*
*************************************
file: loopback.c

Define loopback.


Loopback of a register reg gives the first
equation which rhs depends 
(in one or more steps) from reg.



Define order of instruction in bddvalue.

Initialize loopback to NULL.

if (bddvalue of rhs < bddvalue of lhs)
{
  no updating, return;
}


if (loopback of rhs is NULL)
 {
   if (loopback of lhs is NULL)
     {
       loopback of rhs = lhs;

     }
   else
     {
       loopback of rhs = loopback of lhs;

     }
 }
else 
  {
    if (loopback of lhs is not NULL)
      {
          if ( (bdddvalue of loopback of lhs)
                <
               (bddvalue of loopback of rhs)
             )
          { loopback of rhs = loopback of lhs; }

      }


  }

*************************************
*/


#include "local.h"



#define dbg_flag 0 /* normal value: 0  */






static void* get_loopback_reg();

static void append_loopback(void *prg_reg);

static void reset_loopback_ord(prghdr *p);

static void compose_loopback(prghdr *p, void *prg_reg);

static void ifte_loopback(prghdr *p, void *prg_reg);

static void prj_loopback(prghdr *p, void *prg_reg);

static void join_loopback(prghdr *p, void *prg_reg);

static void lhs_rhs_loopback(prghdr *p, void *lhs_reg, void *rhs_reg);






void define_loopback(prghdr* p)
{
/* dcl */
void* prg_reg;

int update_found;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: begin: program = 0x%x", count, p);
#endif



reset_loopback_ord(p);

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: loopback, ord: initialized", count);
#endif


/*  first instruction  */
prg_reg = p -> prg;


 while (prg_reg != NULL)
  {

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: begin while prg_reg = 0x%x", count, prg_reg);
#endif


       switch( (int) regfield(prg_reg, op_code) )
          {


           case tk_ifte:

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: ifte", count);
#endif


                    ifte_loopback(p, prg_reg);



                     break;




           case tk_compose:

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: compose", count);
#endif

 
                   compose_loopback(p, prg_reg);
                

                   break;





           case tk_prj:

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: prj", count);
#endif

 
                   prj_loopback(p, prg_reg);
                

                   break;






           case tk_join:

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: join", count);
#endif

 
                   join_loopback(p, prg_reg);
                

                   break;




           case tk_exec:

#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: exec", count);
#endif

/* nothing needs to be done */

                 

                   break;





           default:  /* do nothing on query, xarg, boole */

/* Note: no regs on rhs of xarg or boole  */
          
                     break;

          }




       prg_reg = regfield(prg_reg, ptr_down);


  }   /* while */





#if (dbg_flag > 100)
printf("\ndefine_loopback:%u: end: program = 0x%x", count, p);
#endif


}   /* prg  */












void lhs_rhs_loopback(prghdr *p, void *lhs_reg, void *rhs_reg)
{
/* dcl */
unsigned int rhs_stratum;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nlhs_rhs_loopback:%u: begin: prg 0x%x, lhs_reg 0x%x, rhs_reg 0x%x, rhs.loopback = 0x%x", 
count, 
p, lhs_reg, rhs_reg,
regfield(rhs_reg, loopback)
);
#endif


 if ( regfield(lhs_reg, bddvalue)
       >
      regfield(rhs_reg, bddvalue)
     )
            /* no update */
   {
     return;
   } 



/*  
    regfield(lhs_reg, bddvalue)
     <=
    regfield(rhs_reg, bddvalue)
*/

if ( regfield(rhs_reg, loopback) == NULL )
/* first occurrence of rhs_reg on rhs */
   {
     if ( regfield(lhs_reg, loopback) == NULL )
     /* lhs_reg never occurred on the rhs so far */
       {
         regfield(rhs_reg, loopback) = lhs_reg;    
       }
     else  /* lhs_reg already occurred on the rhs */
       {
         regfield(rhs_reg, loopback) = regfield(lhs_reg, loopback);
       }


   }
else  /* rhs_reg already occurred on rhs  */
   {
     if ( regfield(lhs_reg, loopback) != NULL )
     /* lhs_reg already occurred on the rhs  */
       {
           if ( regfield(regfield(lhs_reg, loopback), bddvalue)
                <
                regfield(regfield(rhs_reg, loopback), bddvalue)
              )
            /* update loopback rhs */
            {
               regfield(rhs_reg, loopback) = regfield(lhs_reg, loopback); 
            } 
       }



   }  /* else */


/*  regfield(lhs_reg, loopback) != NULL  */


#if (dbg_flag > 100)
printf("\nlhs_rhs_loopback:%u: loopback of rhs 0x%x = 0x%x", 
count, 
rhs_reg, 
regfield(rhs_reg, loopback)
);
#endif


}  /*  lhs_rhs_loopback   */







void ifte_loopback(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, ifte_word0));

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, ifte_word0 + 1));

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, ifte_word0 + 2));


}  /*  update_ifte_stratum   */


   




void compose_loopback(prghdr *p, void *prg_reg)
{
/* dcl */



/* prg */

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, compose_word0));

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, compose_word0 + 2));


}  /*  update_compose_stratum  */










void prj_loopback(prghdr *p, void *prg_reg)
{
/* dcl */


/* prg */

lhs_rhs_loopback(p, prg_reg, regfield(prg_reg, prj_word0));



}  /*  update_ifte_stratum   */










void join_loopback(prghdr *p, void *prg_reg)
{
/* dcl */
unsigned int i;



/* prg */




           
                    for (
i = 0; 
i < (unsigned int) regfield(prg_reg, ordlist_word0);
i++
)
                       {

lhs_rhs_loopback(
p, 
prg_reg, 
vtxeval(regfield(prg_reg, ordlist_word0 + 1), i)
);

                       } /* for */


}  /*  update_join_stratum   */








/* set loopback to NULL and defines eq. order */

void reset_loopback_ord(prghdr *p)
{
/* dcl */
void *prg_reg;
unsigned int i = 0;

/* prg */


prg_reg = p -> prg;

 while (prg_reg != NULL)
  {
 
       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

           case tk_xarg:
           case tk_boolean:
           case tk_ifte:
           case tk_compose:
           case tk_join:
           case tk_prj:
           case tk_exec:


                        regfield(prg_reg, loopback) = NULL;
                        regfield(prg_reg, bddvalue) = (void *) i;
                        i++;


                        break;



           default:  /* do nothing on query and paddings */
          
                     break;

          }




       prg_reg = vtxeval(regeval(prg_reg), ptr_down);


  }   /* while */





}   /*  reset_loopback_ord  */














/* 
Insert loopback instructions
*/

void insert_loopback(prghdr *p)
{
/* dcl */
void *prg_reg, *next_reg;





#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: begin: p = 0x%x", 
count, p
);
#endif


prg_reg = p -> prg;


 while (prg_reg != NULL)
  {
 
#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: begin while prg_reg = 0x%x", 
count, prg_reg
);
#endif

next_reg = regfield(prg_reg, ptr_down);

#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: next_reg = 0x%x", 
count, next_reg
);
#endif

       switch( (int) vtxeval(regeval(prg_reg), op_code) )
          {

           case tk_ifte:
           case tk_compose:
           case tk_xarg:
           case tk_boolean:
           case tk_prj:
           case tk_join:
           case tk_exec:


#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: prg_reg = 0x%x, loopback = 0x%x", 
count, 
prg_reg,
regfield(prg_reg, loopback)
);
#endif

                  if ( regfield(prg_reg, loopback) != NULL )
                    /* check */
                    {

#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: eq.adr = %u, back.adr = %u", 
count, 
regfield(prg_reg, bddvalue),
regfield(regfield(prg_reg, loopback), bddvalue)
);
#endif

                       if ( regfield(regfield(prg_reg, loopback), bddvalue)
                            <=
                            regfield(prg_reg, bddvalue)
                          )
                        /* insert loopback */
                        {

#if (dbg_flag > 100)
printf("\ninsert_loopback:%u: append loopback to prg_reg = 0x%x", 
count, 
prg_reg
);
#endif

                            append_loopback(prg_reg);
       
                        }
                      

                    }

            


                        break;



           default:  /* do nothing on xarg, boole, query and paddings */
          
                     break;

          }   /*  switch */




       prg_reg = next_reg;


  }   /* while */



}   /* insert_loopback  */











/*
HYP: regfield(prg_reg, ptr_down) != NULL
*/
 

void append_loopback(void *prg_reg)
{
/* dcl */
void *loop_reg;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif


/* prg  */

#if (dbg_flag > 100)
printf("\nappend_loopback:%u: begin: prg_reg = 0x%x", 
count, prg_reg
);
#endif


loop_reg = get_loopback_reg();

regfield(loop_reg, loopback_word0) = regfield(prg_reg, loopback);


/*  append loop_reg to prg_reg */

append_newreg(prg_reg, loop_reg);


#if (dbg_flag > 100)
printf("\nappend_loopback:%u: end: prg_reg = 0x%x", 
count, prg_reg
);
#endif

}  /*  append_loopback  */






void* get_loopback_reg()
{
/* dcl */
void *loop_reg;


/* prg */

loop_reg = get_prgvtx(1);
regeval(loop_reg) = get_prgvtx(size_loopback);
regfield(loop_reg, op_code) = (void *) tk_loopback;

return (loop_reg);

}


