/*
*************************************
file: term.c


Read term from file set in lexan.

Peturns pointer to array of void*
containing token codes or string pointers.
Returns NULL if there is no term to read (eof).
*************************************
*/


#include "local.h"



#define dbg_flag 0  /* normal value: 0  */
 



void* read_new_term()
{
/* dcl */

void* p[2];  /*  token  */



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* prg  */


#if (dbg_flag > 100)
printf("\n\nread_new_term:%u: ckp 100", count);
#endif



switch (read_new_token(p))
    {



       case tk_ifte: /* if then else */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 300: op = ifte", count);
#endif
                    
                     return (read_ifte(p));


       case tk_compose: /* compose */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 350: op = compose", count);
#endif
                    
                     return (read_compose(p));







     case tk_xarg: /* varx  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 600: op = xarg", count);
#endif
                    
                     return (read_xarg(p));



       case tk_boolean: /* boole  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 500: op = boole", count);
#endif
                    
                     return (read_boole(p));





       case tk_xlist: /* xlist */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 400: op = xlist", count);
#endif
                    
                     return (read_xlist(p));



       case tk_reglist: /* reglist */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = reglist", count);
#endif
                    
                     return (read_reglist(p));







       case tk_join: /* join  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: op = join", count);
#endif
                    
                     return (read_join(p));







     case tk_prj: /* prj  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: op = prj", count);
#endif
                    
                     return (read_prj(p));







       case tk_isone:  /* query isone  */
       case tk_eval: /* eval  */
       case tk_satasg:  /* satasg  */
       case tk_C:  /* C function */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 410: query op = %c", count, p[0]);
#endif

                     return (read_query(p));




       case tk_ordlist: /* ordlist */

/* Note: there is at most one ordlist in a bsi program */
/* An ordlist has size > 0  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = ordlist", count);
#endif
                    
                     return (read_ordlist(p));





       case tk_exec: /* exec */


#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = ordlist", count);
#endif
                    
                     return (read_exec(p));




       case tk_eof:  /* EOF: no term to read  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 200: found EOF", count);
#endif

                     return (NULL);






       default:  /* error: unknwon term */

                 printf("\nparse error in file %s line %u: unknown term",
                          lexan_fname(), lex_line_number());

                 exit (1);


    }  /*  switch  */




}   /* prg  */

















