/*
*************************************
file: upload.c

Print prgvtx on file
*************************************
*/


#include "local.h"



#define dbg_flag 1000  /* normal value: 0  */
#define longprint 0  /* normal value: 0  */
#define print_dump 0   /* normal value: 0  */


void fprint_prgvtx(FILE* fileptr, void* p_lab)
{
/* p_lab: pointer to prgvtx */

/* dcl  */
unsigned int i;
void *p;  /* prgvtx */

/* prg  */


#if (dbg_flag > 0)
printf("\nfprint_prgvtx: begin: print reg 0x%x", p_lab);
#endif


/* note p_lab is a reg */


p =  regeval(p_lab);

#if (dbg_flag > 0)
printf("\nfprint_prgvtx: opcode: %c", ((void* *) p)[op_code] );
#endif


       switch ( (unsigned int) ((void* *) p)[op_code] )
          {

              case tk_ifte:
 
                      fprintf(fileptr, "ifte %s %s %s %s\n",
vtxeval(p_lab, 1),
vtxeval(vtxeval(p, ifte_word0), 1),
vtxeval(vtxeval(p, ifte_word0 + 1), 1),
vtxeval(vtxeval(p, ifte_word0 + 2), 1)
);

#if (longprint > 10)
fprintf(fileptr, "/* vtx=0x%x  up=0x%x  down=0x%x  loopback=0x%x */\n",
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down],
((void* *) p)[loopback]
);
#endif

#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_ifte);
fprintf(fileptr, " */\n");
#endif



                     break;


              case tk_compose:
 
                      fprintf(fileptr, "compose %s %s %s %s\n",
vtxeval(p_lab, 1),
vtxeval(vtxeval(p, compose_word0), 1),
vtxeval(vtxeval(p, compose_word0 + 1), 1),
vtxeval(vtxeval(p, compose_word0 + 2), 1)
);

#if (longprint > 10)
fprintf(fileptr, "/* vtx=0x%x  up=0x%x  down=0x%x  loopback=0x%x */\n",
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down],
((void* *) p)[loopback]
);
#endif


#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_compose);
fprintf(fileptr, " */\n");
#endif

 
                        break;










                   case tk_xarg:



                      fprintf(fileptr, "xarg %s %u\n",
vtxeval(p_lab, 1),
vtxeval(p, xarg_word0)
);

#if (longprint > 10)
fprintf(fileptr, "/* vtx=0x%x  up=0x%x  down=0x%x  loopback=0x%x */\n",
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down],
((void* *) p)[loopback]
);
#endif



#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_xarg);
fprintf(fileptr, " */\n");
#endif

 
                        break;











                    case tk_boolean:


                      fprintf(fileptr, "boole %s %u\n",
vtxeval(p_lab, 1),
vtxeval(p, boole_word0)
);

#if (longprint > 10)
fprintf(fileptr, "/* vtx=0x%x  up=0x%x  down=0x%x  loopback=0x%x */\n",
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down],
((void* *) p)[loopback]
);
#endif

#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_boole);
fprintf(fileptr, " */\n");
#endif



                        break;










           case tk_isone:  /* query isone  */
           case tk_eval: /* eval  */
           case tk_satasg:  /* satasg  */
           case tk_C:  /* C function */




                      fprintf(fileptr, "%c %s \"%s\"\n",
((void* *) p)[op_code],
vtxeval(vtxeval(p, query_word0), 1),
vtxeval(p, query_word0 + 1)
);

#if (longprint > 10)
fprintf(fileptr, "/* id 0x%x vtx 0x%x up 0x%x down 0x%x */\n",
p_lab,
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down]
);
#endif


#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_query);
fprintf(fileptr, " */\n");
#endif

 
                        break;







            case tk_xlist:  

                      fprintf(fileptr, "xlist %s (",
vtxeval(p_lab, 1)
);


              
                    for (i = 0; i < ((unsigned int ) ((void* *) p)[xlist_word0]); i++)
                       {
fprintf(fileptr, "%u ",
((unsigned int *) ((void* *) p)[xlist_word0 + 1])[i]
 );
                       } /* for */

                      fprintf(fileptr, ")\n");


#if (longprint > 10)
fprintf(fileptr, "/* id 0x%x vtx 0x%x up 0x%x down 0x%x */\n",
p_lab,
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down]
);
#endif


#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_xlist);
fprintf(fileptr, " */\n");
#endif

 
                        break;






            case tk_reglist:  

                      fprintf(fileptr, "reglist %s (",
vtxeval(p_lab, 1)
);


              
                    for (i = 0; i < (unsigned int) vtxeval(p, reglist_word0); i++)
                       {
fprintf(fileptr, "%s ",
vtxeval(vtxeval(regfield(p_lab, reglist_word0 + 1), i), 1)
 );
                       } /* for */

                      fprintf(fileptr, ")\n");


#if (longprint > 10)
fprintf(fileptr, "/* id 0x%x vtx 0x%x up 0x%x down 0x%x */\n",
p_lab,
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down]
);
#endif

#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_reglist);
fprintf(fileptr, " */\n");
#endif


 
                        break;



            case tk_ordlist:  

                      fprintf(fileptr, "ordlist %s (",
vtxeval(p_lab, 1)
);


              
                    for (i = 0; i < ((unsigned int) ((void* *) p)[ordlist_word0]); i++)
                       {
fprintf(fileptr, "%u ",
((unsigned int *) ((void* *) p)[ordlist_word0 + 1])[i]
 );

                       } /* for */

                      fprintf(fileptr, ")\n");


#if (longprint > 10)
fprintf(fileptr, "/* id 0x%x vtx 0x%x up 0x%x down 0x%x */\n",
p_lab,
p,
((void* *) p)[ptr_up],
((void* *) p)[ptr_down]
);
#endif


#if (print_dump > 10)
fprintf(fileptr, "/* ");
dump_reg(fileptr, p_lab, size_ordlist);
fprintf(fileptr, " */\n");
#endif


 
                        break;





           case tk_begin:  
           case tk_end:
                          /* These are internal instructions
                             only used for padding.
                             No printing 
                           */

                         break;











       case tk_and:
       case tk_or:
       case tk_xor:
       case tk_eq:
       case tk_nor:
       case tk_nand:
       case tk_imp:



#if (dbg_flag > 0)
printf("\nfprint_prgvtx: binary boole op");
#endif


                      fprintf(fileptr, "%c %s %s %s\n",
((void* *) p)[op_code],
vtxeval(p_lab, 1),
vtxeval(vtxeval(p, size_0op), 1),
vtxeval(vtxeval(p, size_0op + 1), 1)
);

  


                  break;




         case tk_not:



#if (dbg_flag > 0)
printf("\nfprint_prgvtx: not");
#endif



                      fprintf(fileptr, "%c %s %s\n",
((void* *) p)[op_code],
vtxeval(p_lab, 1),
vtxeval(vtxeval(p, size_0op), 1)
);

  


                  break;



       case tk_exists: 
       case tk_forall: 



#if (dbg_flag > 0)
printf("\nfprint_prgvtx: exists/forall");
#endif



                      fprintf(fileptr, "%c %s %u %s\n",
((void* *) p)[op_code],
vtxeval(p_lab, 1),
vtxeval(p, size_0op),
vtxeval(vtxeval(p, size_0op + 1), 1)
);

  


                  break;








                 default:  /* error  */
                          printf("\nfprint_prgvtx: error: unknown prog. instruction");
                          exit (1);
        

          }  /* switch */



#if (dbg_flag > 0)
printf("\nfprint_prgvtx: ckp 1100 (end)");
#endif



}  /*  prg  */






void dump_reg(FILE* fileptr, void* p, unsigned int size)
{
/* dcl */
unsigned int i;

/* prg */
fprintf(fileptr, "dump id = 0x%x, vtx = *id = 0x%x:\n",
p, *((void**) p));

for (i = 0; i < size; i++)
   fprintf(fileptr, "vtx[%u] = 0x%x ", i, (*((void***) p))[i]);


}  /*  dump_reg  */

















