/*
*************************************
file: term.c


Read term from file set in lexan.

Peturns pointer to array of void*
containing token codes or string pointers.
Returns NULL if there is no term to read (eof).
*************************************
*/


#include "local.h"



#define dbg_flag 0    /* normal value: 0  */
 



void* read_new_term()
{
/* dcl */

void* p[2];  /*  token  */

unsigned int arity;
void *result;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* prg  */


#if (dbg_flag > 100)
printf("\n\nread_new_term:%u: ckp 100", count);
#endif



#if 0
/* read '(' */
read_new_token(p);
if (p[0] != (void *) tk_leftpar)
/* error  */
  {
       printf("\nerror in file %s line %u: '(' expected",
               lexan_fname(), lex_line_number()
             );
       exit (1000);
  }
#endif




/* ok '(' found */

while (1)
{

switch (read_new_token(p))
    {


       case tk_leftpar:  /* begin term: go to next ite */

                         continue;



       case tk_ifte: /* if then else */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 300: op = ifte", count);
#endif
                    
 result = read_boole_op(p, &arity);
                   
if (arity != 3)
/* error */
{
 printf("\nparse error in file %s line %u: 3 rhs args expected for ifte",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );

                



       case tk_and:
       case tk_or:
       case tk_xor:
       case tk_eq:
       case tk_nor:
       case tk_nand:
       

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: boolean operator: code: %c", count, p[0]);
#endif
 
                  
 result = read_boole_op(p, &arity);
                   
if (arity < 2)
/* error */
{
 printf("\nparse error in file %s line %u: at least 2 args expected",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );







       case tk_not:


#if (dbg_flag > 1000)
printf("\nread_new_term:%u: boolean operator: code: %c", count, p[0]);
#endif

                     result = read_boole_op(p, &arity);
                   
if (arity != 1)
/* error */
{
 printf("\nparse error in file %s line %u: 1 rhs arg expected for not",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );








       case tk_imp:


#if (dbg_flag > 1000)
printf("\nread_new_term:%u: boolean operator: code: %c", count, p[0]);
#endif
                    
                     result = read_boole_op(p, &arity);
                   
if (arity != 2)
/* error */
{
 printf("\nparse error in file %s line %u: 2 rhs args expected for imp",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );








       case tk_compose: /* compose */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 350: op = compose", count);
#endif
                    
                     return (read_compose(p));







     case tk_xarg: /* varx  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 600: op = xarg", count);
#endif
                    
                     return (read_xarg(p));



       case tk_boolean: /* boole  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 500: op = boole", count);
#endif
                    
                     return (read_boole(p));







       case tk_exists: 
       case tk_forall: 

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: exists, forall", count);
#endif
                    
                     return (read_quantifiers(p));





       case tk_xlist: /* xlist */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 400: op = xlist", count);
#endif
                    
                 
 result = read_xlist(p, &arity);
                   
if (arity < 1)
/* error */
{
 printf("\nparse error in file %s line %u: at least 1 rhs arg expected",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );







       case tk_reglist: /* reglist */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = reglist", count);
#endif
                    
                   
 result = read_boole_op(p, &arity);
                   
if (arity < 1)
/* error */
{
 printf("\nparse error in file %s line %u: at least 1 rhs arg expected",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );







       case tk_join: /* join */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = join", count);
#endif
                    
                   
 result = read_boole_op(p, &arity);
                   
if (arity < 2)
/* error */
{
 printf("\nparse error in file %s line %u: at least 2 rhs args expected",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );









     case tk_prj: /* prj  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 600: op = prj", count);
#endif
                    
                     return (read_prj(p));







       case tk_isone:  /* query isone  */
       case tk_eval: /* eval  */
       case tk_satasg:  /* satasg  */
       case tk_C:  /* C function */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 410: query op = %c", count, p[0]);
#endif

                     return (read_query(p));




       case tk_ordlist: /* ordlist */

/* Note: there is at most one ordlist in a bsi program */
/* An ordlist has size > 0  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 450: op = ordlist", count);
#endif
                    
       
 result = read_ordlist(p, &arity);
                   
if (arity < 1)
/* error */
{
 printf("\nparse error in file %s line %u: at least 1 rhs arg expected",
                          lexan_fname(), lex_line_number());

                 exit (1);


}

                     return ( result );







      case tk_exec:  /* exec */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 410: exec = %c", count, p[0]);
#endif

                     return (read_exec(p, &arity));







       case tk_eof:  /* EOF: no term to read  */

#if (dbg_flag > 1000)
printf("\nread_new_term:%u: ckp 200: found EOF", count);
#endif

                     return (NULL);






       default:  /* error: unknwon term */

                 printf("\nread_new_term: parse error in file %s line %u: unknown term",
                          lexan_fname(), lex_line_number());

                 exit (1);


    }  /*  switch  */



}  /* while */




}   /* prg  */

















