/*
*************************************
file: bop.c

Map binder operators (exists, forall) to bsh.
term points to a term: 
(def A (binder X q))

id X: array, record, dollar
id q: array, record, dollar, bfun.

HYP: type checking done.
vctsize well defined.


Define component i, arg j.


Note: arg 0 (j == 0) should be array, record or dollar.
This is checked at type checking (syntax).
*************************************
*/


#include "local.h"



#define dbg_flag 0   /* normal value: 0  */




void prt_binder_arg(FILE* fileptr, void* term, unsigned int i, unsigned int j)
{
/* dcl */

void *arg_term, *rhs_term;
void* array_def;
char* array_name;
unsigned int array_entry;
char *lhs_name;

unsigned int lhs_idcount;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif



/* prg  */


#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: begin: term 0x%x: cmp i = %u, arg j = %u:\n", count, 
term,
i,
j
);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif


lhs_name = (char *) 
               vtxeval(
                  vtxeval(vtxeval(term, op_word0), op_word0),
                  lexid_string
                      );


lhs_idcount = (unsigned int) 
               vtxeval(
                  vtxeval(vtxeval(term, op_word0), op_word0),
                  lexid_string
                      );

arg_term = vtxeval(
             vtxeval(term, op_word0 + 1),
             op_word0 + j
                  );



#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: vctsize = %u \n", count, 
vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
);
#endif



/*
******************************
There is an object to print only if
i < obj size.
*******************************
*/


if (i >= (unsigned int) 
         vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
   )
/* nothing to be defined */
{
return;
}

/*
*****************

i < (unsigned int) 
         vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
******************
*/



#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: arg_term 0x%x: cmp i = %u, arg j = %u\n", count, 
arg_term,
i,
j
);
print_term(stderr, arg_term);
fprintf(stderr, "\n");
#endif




#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: arg_term op_code = %u\n", count, 
vtxeval(arg_term, op_code)
);
#endif



/* check for keywords: b0, b1 */

switch ( (unsigned int) vtxeval(arg_term, op_code)  )
   {

     case tk_b0:
           fprintf(fileptr, " b0");
           return;


     case tk_b1:
           fprintf(fileptr, " b1");
           return;


     case tk_id:  /* next switch */
             break;



    default: /* error   */

fprintf(stderr, "\nprt_binder_arg: Error: unexpected token in arg (%u, %u) in term:\n", i, j);
print_term(stderr, term);

            exit (1);

   }






#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: id found: id type: %u", 
count,
vtxeval(vtxeval(arg_term, op_word0), lexid_type)
);
#endif



/* id found   */

switch ( (unsigned int) vtxeval(vtxeval(arg_term, op_word0), lexid_type) )
   {




     case id_bfun:


   if ( (unsigned int) 
        vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
        > 1
      )
   /* use prj */
     {

   fprintf(fileptr, " _prj_%u_%s_%u",
 vtxeval(
                  vtxeval(vtxeval(term, op_word0), op_word0),
                  lexid_idcount
                      ),
vtxeval(vtxeval(arg_term, op_word0), lexid_string),
i
);
    }

 else  /* no prj  */
    {
   fprintf(fileptr, " %s",
vtxeval(vtxeval(arg_term, op_word0), lexid_string)
);


    }



                 return;




      case id_array:   /* get component i */
   
                   fprintf(fileptr, " %sa%u%s",
signe(),
i,
vtxeval(vtxeval(arg_term, op_word0), lexid_string)
);

                     return;











      case id_record:   /* get component j */




array_def = record_entry(
vtxeval(vtxeval(arg_term, op_word0), lexid_def), 
i,
&array_entry
);

array_name = (char *) 
             vtxeval(
               vtxeval(vtxeval(array_def, op_word0), op_word0),
               lexid_string
                    );


                  fprintf(fileptr, " %sa%u%s",
signe(),
array_entry,
array_name
);


                     return;











     
      case id_dollar:   /* size 1, get array entry */


#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: id_dollar", 
count
);
#endif



rhs_term = vtxeval(vtxeval(vtxeval(arg_term, op_word0), lexid_def), 
                   op_word0 + 1);

#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: id_dollar: rhs_term = 0x%x: ", 
count,
rhs_term
);
print_term(stderr, rhs_term);
fprintf(stderr, "\nprt_binder_arg:%u: num: %u ", 
count,
vtxeval(vtxeval(rhs_term, op_word0 + 1),  op_word0)
);
fprintf(stderr, "\nprt_binder_arg:%u: string: %s", 
count,
vtxeval(
   vtxeval(vtxeval(rhs_term, op_word0), op_word0),
   lexid_string
       )
);
#endif


               fprintf(fileptr, " %sa%u%s",
signe(),
vtxeval(vtxeval(rhs_term, op_word0 + 1),  op_word0),
vtxeval(
   vtxeval(vtxeval(rhs_term, op_word0), op_word0),
   lexid_string
       )
);


                    return;







    default: /* error   */

fprintf(stderr, 
"Error: id expected in arg (%u, %u) in term:\n", 
i, j);
print_term(stderr, term);

            exit (1);





   }




#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_arg:%u: end", count);
#endif


}  /*  prt_binder_arg  */

















