/*
*************************************
file: prbndvar.c

Map binder operators (exists, forall) to bsh.
term points to a term: 
(def A (binder X q))

id X: array, record, dollar
id q: array, record, dollar, bfun.

HYP: type checking done.
vctsize well defined.


Define component i, arg j.


Note: arg 0 (j == 0) should be array, record or dollar.
This is checked at type checking (syntax).

Print xlist for X.
*************************************
*/


#include "local.h"



#define dbg_flag 0   /* normal value: 0  */




void prt_binder_var(FILE* fileptr, void* term, unsigned int i, unsigned int j)
{
/* dcl */

void *arg_term, *rhs_term;
void* array_def;
char* array_name;
unsigned int array_entry;
char *lhs_name;

unsigned int lhs_idcount;

#if (dbg_flag > 0)
static int count = 0;
count++;
#endif



/* prg  */


#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: begin: term 0x%x: cmp i = %u, arg j = %u\n", count, 
term,
i,
j
);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif

if (j != 0)
/* error */
{

fprintf(stderr, "\nprt_binder_var: calling arg j = %u, expected arg j = 0\n",
j);

exit (1);

}


lhs_name = (char *) 
               vtxeval(
                  vtxeval(vtxeval(term, op_word0), op_word0),
                  lexid_string
                      );


lhs_idcount = (unsigned int) 
               vtxeval(
                  vtxeval(vtxeval(term, op_word0), op_word0),
                  lexid_string
                      );

arg_term = vtxeval(
             vtxeval(term, op_word0 + 1),
             op_word0 + j
                  );



#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: vctsize = %u \n", count, 
vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
);
#endif



/*
******************************
There is an object to print only if
i < obj size.
*******************************
*/


if (i >= (unsigned int) 
         vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
   )
/* nothing to be defined */
{
return;
}

/*
*****************

i < (unsigned int) 
         vtxeval(vtxeval(arg_term, op_word0), lexid_vctsize)
******************
*/



#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: arg_term 0x%x: cmp i = %u, arg j = %u\n", count, 
arg_term,
i,
j
);
print_term(stderr, arg_term);
fprintf(stderr, "\n");
#endif




#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: arg_term op_code = %u\n", count, 
vtxeval(arg_term, op_code)
);
#endif



/* check for keywords: b0, b1 */

switch ( (unsigned int) vtxeval(arg_term, op_code)  )
   {

     case tk_b0:
     case tk_b1:

fprintf(stderr, "\nprt_binder_var: found b0/b1, expected: array, record, dollar\n");

exit (1);



     case tk_id:  /* next switch */
             break;



    default: /* error   */

fprintf(stderr, "\nprt_binder_var: Error: unexpected token in arg (%u, %u) in term:\n", i, j);
print_term(stderr, term);

            exit (1);

   }






#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: id found: id type: %u", 
count,
vtxeval(vtxeval(arg_term, op_word0), lexid_type)
);
#endif



/* id found   */

switch ( (unsigned int) vtxeval(vtxeval(arg_term, op_word0), lexid_type) )
   {




     case id_bfun:


fprintf(stderr, "\nprt_binder_var: found bfun, expected: array, record, dollar\n");

exit (1);




      case id_array:   /* xlist for array  */
   
                   fprintf(fileptr, " %sX%s",
signe(),
vtxeval(vtxeval(arg_term, op_word0), lexid_string)
);

                     return;











      case id_record:   /* xlist for record */



                   fprintf(fileptr, " %sX%s",
signe(),
vtxeval(vtxeval(arg_term, op_word0), lexid_string)
);

                     return;









     
      case id_dollar:   /* size 1, get array entry */


#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: id_dollar", 
count
);
#endif



                   fprintf(fileptr, " %sxd%s",
signe(),
vtxeval(vtxeval(arg_term, op_word0), lexid_string)
);

                     return;









    default: /* error   */

fprintf(stderr, 
"Error: id expected in arg (%u, %u) in term:\n", 
i, j);
print_term(stderr, term);

            exit (1);





   }




#if (dbg_flag > 10)
fprintf(stderr, "\nprt_binder_var:%u: end", count);
#endif


}  /*  prt_binder_var  */

















