/*
*************************************
file: eval.c


Eval term representing an expression.
Error if term is not an expression
or some id in expression is not
defined (via a const).


Return expression value.

*************************************
*/


#include "local.h"

#include <math.h>



#define dbg_flag 0  /* normal value: 0  */



unsigned int eval_expr(void* term)
{
/* dcl */

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* prg  */



#if (dbg_flag > 100)
printf("\nprint_term:%u: begin: term = 0x%x: op_code = %c = %u", 
count, term,
vtxeval(term, op_code),
vtxeval(term, op_code)
);
#endif



      switch ( (unsigned int) vtxeval(term, op_code) )
            {



       case tk_id:  /* id  */


#if (dbg_flag > 100)
printf("\nprint_term:%u: begin id:", count);
printf("\nvtxeval(term, op_word0) = 0x%x", 
vtxeval(term, op_word0)
);
printf("\nidarray(vtxeval(term, op_word0)) = 0x%x", 
idarray(vtxeval(term, op_word0))
);
printf("\nvtxeval(idarray(vtxeval(term, op_word0)), lexid_string) = 0x%x",
vtxeval(idarray(vtxeval(term, op_word0)), lexid_string)
);
printf("\nid = %s",
vtxeval(idarray(vtxeval(term, op_word0)), lexid_string)
);
#endif


/* check if id has been defined with a const */

if (

vtxeval(idarray(vtxeval(term, op_word0)), lexid_type)
!=
(void *) id_const

   )
/* error id has not been declared with a const  */
{

fprintf(stderr, 
"\neval_expr: error id %s has not been defined with a const in term:\n",
vtxeval(idarray(vtxeval(term, op_word0)), lexid_string)
);

print_term(stderr, term);

exit (1);

}


/* ok id const defined */



        return (
(unsigned int)
vtxeval(idarray(vtxeval(term, op_word0)), lexid_value)
               );








      case tk_num:  /* id  */

#if (dbg_flag > 100)
printf("\nprint_term:%u: num: %u", count, vtxeval(term, op_word0));
#endif

        return (
(unsigned int)
vtxeval(term, op_word0)
               );







      case tk_ap:  /* ap  */

  
#if (dbg_flag > 100)
printf("\nprint_term:%u: found tk_ap: ", count);
#endif


/* Parser guarantees: 
   (unsigned int) vtxeval(term, op_args) > 1 
*/



          return ( apterm_eval(term) );




       default:  /* error:  */


#if (dbg_flag > 100)
printf("\nprint_term:%u: keyword: %s", count, 
keywstring((unsigned int) vtxeval(term, op_code))
);
#endif



fprintf(stderr, 
"\neval_expr: error: unexpected token in term:\n");

print_term(stderr, term);

exit (1);




            }  /*  switch */



}   /* eval_expr  */














/*
term is (op ....  )
*/


unsigned int apterm_eval(void* term)
{
/* dcl */

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* prg  */



#if (dbg_flag > 100)
printf("\nprint_term:%u: begin: term = 0x%x: op_code = %c = %u", 
count, term,
vtxeval(term, op_code),
vtxeval(term, op_code)
);
#endif



      switch ( (unsigned int) vtxeval(vtxeval(term, op_word0), op_code) )
            {



       case tk_ADD:  /* +  */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

eval_expr( vtxeval(term, op_word0 + 1) )
+
eval_expr( vtxeval(term, op_word0 + 2) )
       );






       case tk_SUB:  /* -  */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

eval_expr( vtxeval(term, op_word0 + 1) )
-
eval_expr( vtxeval(term, op_word0 + 2) )
       );







       case tk_MULT:  /*  *  */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

eval_expr( vtxeval(term, op_word0 + 1) )
*
eval_expr( vtxeval(term, op_word0 + 2) )
       );








       case tk_DIV:  /* /   */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

eval_expr( vtxeval(term, op_word0 + 1) ) 
/
eval_expr( vtxeval(term, op_word0 + 2) )
       );















       case tk_MOD:  /* % */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

eval_expr( vtxeval(term, op_word0 + 1) )
%
eval_expr( vtxeval(term, op_word0 + 2) )
       );








       case tk_EXP:  /* ^ */



if ( (unsigned int) vtxeval(term, op_args) != 3 )
/* error */
{

fprintf(stderr, 
"\neval_expr: error: found %u args instead than 2 in term:\n",
vtxeval(term, op_args) - 1
);

print_term(stderr, term);

exit (1);


}



/* OK  */



return (

(unsigned int)
pow(
eval_expr( vtxeval(term, op_word0 + 1) ),
eval_expr( vtxeval(term, op_word0 + 2) )
   )
       );









       default:  /* error:  */



fprintf(stderr, 
"\napterm_eval: error: unexpected token in term:\n");

print_term(stderr, term);

exit (1);




            }  /*  switch */



}   /* apterm_eval  */















