/*
*************************************
file: term.c


Read term from file set in lexan.

Peturns pointer to array of void*
containing token codes or string pointers.
Returns NULL if there is no term to read (eof).
*************************************
*/


#include "local.h"



#define dbg_flag 0  /* normal value: 0  */

#define prt_outsize 0  /* normal value: 0 */




void term_trans(FILE *fileptr, void* term)
{
/* dcl */

unsigned int i;


#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* prg  */



#if (dbg_flag > 100)
printf("\n\nterm_trans:%u: begin translation term 0x%x: \n", 
count, 
term
);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif





      switch ( (unsigned int) vtxeval(term, op_code) )
            {



       case tk_id:  /* id  */


#if (dbg_flag > 100)
fprintf(stderr, "\nterm_trans:%u: term 0x%x is id %s", 
count,
term,
vtxeval(idarray(vtxeval(term, op_word0)), lexid_string)
);
#endif


trans_id(fileptr, term);

return;







      case tk_num:  /* id  */

#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is num: %u", 
count, term, vtxeval(term, op_word0));
#endif

            fprintf(fileptr, "%d ", 
                       vtxeval(term, op_word0)
                    );
             return;








      case tk_string:  /* id  */


#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is string: \"%s\"", 
count, term, vtxeval(term, op_word0)
);
#endif

            fprintf(fileptr, "\"%s\" ", 
                       vtxeval(term, op_word0)
                    );
             return;







      case tk_ap:  /* ap  */

  
#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is tk_ap: ", count, term);
#endif


/* Parser guarantees: 
   (unsigned int) vtxeval(term, op_args) > 1 
*/





         switch ((unsigned int) vtxeval(vtxeval(term, op_word0), op_code))
           {

               case tk_ADD:
               case tk_SUB:
               case tk_MULT:
               case tk_DIV:
               case tk_MOD:
               case tk_EXP:

  
#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is an expression with value %u: term:\n", 
count, 
term,
eval_expr(term)
);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif




/* expression */

             fprintf(fileptr, " %u ", eval_expr(term) );

                  return;





             case tk_lambda:


                  
                    lambda(fileptr, term);

                    return;





             case tk_id:


  
#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is\n", 
count, 
term
);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif


                 switch ( (unsigned int)
vtxeval(idarray(
              vtxeval(vtxeval(term, op_word0), op_word0)
                    ), lexid_type)
                        )
                     {  /* id switch */

                      

                  case id_macro:


#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is a macro call", 
count, 
term
);
#endif


                    macro(fileptr, term);

                    return;




                     

                  case id_macro_arg:


#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is a macro arg", 
count, 
term
);
#endif



                 if (
vtxeval(idarray(
              vtxeval(vtxeval(term, op_word0), op_word0)
                    ), lexid_value)

==
(void *) NULL
   )
/* error */
{
fprintf(stderr, 
"\nterm_trans: error: macro_arg id %s undefined in term:\n",

vtxeval(idarray(
              vtxeval(vtxeval(term, op_word0), op_word0)
                    ), lexid_string)
);

print_term(stderr, term);
fprintf(stderr, "\n");

exit (1);


}

/* OK */


fprintf(fileptr, "(");


for (i = 0; i < (unsigned int) vtxeval(term, op_args); i++)
    {

term_trans(fileptr, vtxeval(term, op_word0 + i));

    }


fprintf(fileptr, ")");




                  return;



                default:  /* error  */

fprintf(stderr, 
"\nterm_trans: error: id %s expected to be a macro name in term:\n",

vtxeval(idarray(
              vtxeval(vtxeval(term, op_word0), op_word0)
                    ), lexid_string)
);

print_term(stderr, term);
fprintf(stderr, "\n");

exit (1);



                   }  /* ap.id switch */





               case tk_const:
               case tk_enum:
               case tk_macro:

  
#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is not to be translated: term:\n", count, term);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif



                     return;






/* non expression */

            default:

  
#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is NOT an expression ", count, term);
print_term(stderr, term);
fprintf(stderr, "\n");
#endif




             fprintf(fileptr, "(");

            for (i = 0; 
                 i < (unsigned int) vtxeval(term, op_args);
                 i++
                )
             {
                term_trans(fileptr, vtxeval(term, op_word0 + i));
             }


             fprintf(fileptr, ")");

             return;

           }  /* switch  */



  



       default:  /* keyword  */


#if (dbg_flag > 100)
printf("\nterm_trans:%u: term 0x%x is keyword: %s", 
count, term,
keywstring((unsigned int) vtxeval(term, op_code))
);
#endif

            fprintf(fileptr, "%s ", 
                       keywstring((unsigned int) vtxeval(term, op_code))
                    );



             return;





            }  /*  switch */








}   /* prg  */

















