/*
*************************************
file: readtken.c


read new token from lexfile_ptr

if new and nonkeyword
store it in symbol table T

returns token
*************************************
*/


#include "local.h"


/* To print just return codes set dbg_flag to 25 */
#define dbg_flag 0         /* normal value: 0  */
 


/*  exit on unknown symbol: normal value: > 0  */
#define exit_on_unk 100 




/*  
        p[0] = token_code, 
        p[1] = case
                    (token is an identifier or keyword) then  
                             pointer to bucket in symbol table;
                    
                    (token is a string) then  
                                   pointer to string;

                    (token is a numeral) then
                                   numeral value

                    default: undefined



       returns token_code

with  
        tk_newid when id found was not in symbol table T,
      
        tk_oldid when id found was already in symbol table T


*/





unsigned int read_new_token(void* *p)
{
/* declarations  */

char c;

void* temp;

int tokenval;
unsigned int i;



#if (dbg_flag > 0)
static int count = 0;
count++;
#endif




/* programs  */


#if (dbg_flag > 1000)
printf("\n\nread_new_token:%u: ckp 100", count);
#endif




while (1)
     {


          c = fgetc(lexfile_ptr);


#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 200: read character: %c",
count,
c
);
#endif

                  if (c == EOF)
                    {
                        p[0] = (void *) tk_eof;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 210: eof: return tk_eof",
count
);
#endif

                        return (tk_eof);

                    }


/* c != EOF */


          if ( isspace(c) && (c != '\n') )
                 /*  skip spaces, tabs, formfeeds  */
            {

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 300: skip white space and tabs",
count
);
#endif

 
            }


         else /* c != EOF */

               if (c == '\n')
                  /*  newline   */
                {

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 400: skip newline",
count
);
#endif

                  line_number++;


                 }



        else /* c != EOF */


              if ((c == '%') || (c == '#'))
                 /* comment: skip rest of line */
                 { 

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 500: begin skip line comment",
count
);
#endif


                    c = fgetc(lexfile_ptr);


#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 505: c = %c",
count,
c
);
#endif

                    while ( (c != '\n') && (c != EOF) )
                         {  
#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 510: c = %c",
count,
c
);
#endif

                               c = fgetc(lexfile_ptr); 

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 520: c = %c",
count,
c
);
#endif


                         }

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 530: c = %c",
count,
c
);
#endif


                    /* c == '\n' || c == EOF  */

                    if (c == EOF)
                      {
                        p[0] = (void *) tk_eof;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 540: eof: return tk_eof",
count
);
#endif
                        return (tk_eof);
                      }
               
                    /*   c == '\n'  */
                    line_number++;

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 600: end skip line comment",
count
);
#endif

                 }







          else /* c != EOF */
               if (c == '/')
                  /* may be C style comment is starting */
                  {
/*
**********************************************
begin skip comment
**********************************************
*/

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 700: begin test C comment",
count
);
#endif


 c = fgetc(lexfile_ptr);

                  if (c == EOF)
                    {

                       p[0] = (void *) tk_eof;


#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 740: eof: return tk_eof",
count
);
#endif

                       return (tk_eof);

                    }


 if (c != '*')
    /*  unk sequence  */
    {

        p[1] = (void *) (unsigned int) c;
        ungetc(c, lexfile_ptr);

        p[0] = (void *) tk_unk;


#if (exit_on_unk > 0)
   printf("\nerror: unknown input symbol '%c' at line %u", 
c,
line_number);
   exit (1000);
#endif



#if (dbg_flag > 20)
printf("\nread_new_token:%u: unk: '%c': return tk_unk",
count, c
);
#endif

        return (tk_unk);
       

    }

/*  c == '*' :  begin comment   */

    c = fgetc(lexfile_ptr);



    while (1)
         {

                  if (c == EOF)
                    {


                        write_token_lexcode(p, (unsigned int) tk_eof);


#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1240: eof: return tk_eof",
count
);
#endif


                        return (tk_eof);

                    }

/*  c != EOF  */

                 if (c == '*')
                   /* may be end of comment  */
                   {
                        c = fgetc(lexfile_ptr);

                        if (c == EOF) 
                          { 
                        write_token_lexcode(p, (unsigned int) tk_eof);

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1540: eof: return tk_eof",
count
);
#endif

                        return (tk_eof);
                          }

                        /* c != EOF  */

                        if (c == '/')
                           /*  end of comment  */
                          { break; }



                        /* c != EOF && c != '/'  */

                   }

               /* c != '*' and c != EOF and c != '/'  */

               if (c == '\n') line_number++;

               c = fgetc(lexfile_ptr);

         }    /*  while  */


#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 900: end test C comment",
count
);
#endif


/*
*********************************************
end skip comment
*********************************************
*/

                  }











/*
*******************************
check number (signed integer)
*******************************
*/




         else /* c != EOF */
              if ( isdigit(c) || (c == '-') )
                 /*  signed decimal  */
                 { 

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1000: begin read number",
count
);
#endif

                     ungetc(c, lexfile_ptr);

                     fscanf(lexfile_ptr, "%d", &tokenval);

                     p[0] = (void *) tk_num;
                     p[1] = (void *) tokenval;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1100: found number: p[0] = %u, p[1] = %u: return u%u = d%d",
count,
p[0],
p[1],
tokenval,
tokenval
);
#endif

                     return (tk_num);

                 }





/* !isdigit(c)  */

/*
************************************************
Check if token is a one nonletter char  keyword  
************************************************
*/

     else  /* c != EOF */

     if ( (c == ')') || (c == '(') ) 
        {
            lexbuf[0] = c;
            lexbuf[1] = '\0';

            temp = read_symbtab_bucket(Key_Words_Table, lexbuf);

            p[0] = (void *) read_stbk_tkcode(temp);
            p[1] = temp;


#if (dbg_flag > 20)
printf("\nread_new_token:%u: found one char keyword: '%c': p[0] = %u = %c, p[1] = 0x%x: return p[0]",
count,
c,
p[0], p[0],
p[1]
);
#endif

            return ( (unsigned int) p[0] );

        }



     else  /* c != EOF */
    

/*
**************************************
Check if token is an id possibly a keyword
**************************************
*/

      if ( isalpha(c) || (c == '_') || (c == '.') )
                 /*  c is a letter or _  */
                 { 

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1200: begin read identifier",
count
);
#endif
                     i = 0;
                     while ( isalnum(c) || (c == '_') || (c == '.') )
                          {
#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1210: lexbuf[%u] = %c",
count,
i,
c
);
#endif

                             lexbuf[i] = c;
                             c = fgetc(lexfile_ptr);
                             i++;

                             if (i >= BSIZE)
                                /* error  */
                                {
                                   printf("\nerror line %u: identifier %s... too long", 
line_number,
lexbuf
);
                                    exit (1000);
                                }

                          }  /*  while  */
          
                   /*  i < BSIZE  */

                     lexbuf[i] = '\0';  
                     if (c != EOF) ungetc(c, lexfile_ptr);

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1300: found id: %s",
count,
lexbuf
);
#endif

                     /*  check if id is a keyword  */

                     temp = read_symbtab_bucket(Key_Words_Table, lexbuf);

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1250: temp = 0x%x",
count,
temp
);
#endif


                     if (temp != NULL)
                       /* id not new  */
                        {
                            
                            if (read_stbk_info(temp) == NULL)
                                /* keyword */
                               {
                                 p[0] = (void *) read_stbk_tkcode(temp);
                                 p[1] = temp;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1715: keyword %s: p[0] = %u = %c, p[1] = %u, return %u",
count,
read_stbk_key(temp),
p[0], p[0],
p[1],
p[0]
);
#endif

                            return ((unsigned int) p[0]) ;

                              }


                           else  /* old id  */
                               {

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1610: old id: %s",
count,
lexbuf
);
#endif

                       p[0] = (void *) tk_id;
                       p[1] = temp;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1613: old id: %s: p[0] = %u, p[1] = %u, return %u",
count,
lexbuf,
p[0],
p[1],
tk_oldid
);
#endif

                       return (tk_oldid);

                               }


                        }
                     else  /* temp == NULL  */
                          /*  new id  */
                        {
#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1600: new id: %s",
count,
lexbuf
);
#endif

                          /*  store id in symbol table  */
                           temp = get_new_stbk();
                           write_stbk_key(temp, duplicate_key(lexbuf));
                           write_stbk_tkcode(temp, tk_id);
                           write_stbk_info(temp, get_newid_vtx()); 
                           write_symbtab_bucket(Key_Words_Table, temp);


/* initialize id value to NULL */
*((void* *) read_stbk_info(temp)) = NULL;



/* temp = bucket  */

                    /*  temp != NULL  */
                       
                       p[0] = (void *) tk_id;
                       p[1] = temp;

#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 1603: new id: %s: p[0] = %u, p[1] = %u, return %u",
count,
lexbuf,
p[0],
p[1],
tk_newid
);
#endif

                        return (tk_newid);

                        }  /* else: temp == NULL  */

                 } /* then c is letter or _ or .  */
                 






/*
**************************************************
token is not a number, not an id,  not a keyword
**************************************************
*/



/*
*************************************
check for string
*************************************
*/





         else /* c != EOF */
             if ( (c == '\"') )
              {  /* string follows  till "  */
#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1200: begin read identifier",
count
);
#endif
                i = 0;
                     
                 while ( 1 )
                          {

                c = fgetc(lexfile_ptr);

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1210: lexbuf[%u] = %c",
count,
i,
c
);
#endif

if ((c == EOF) || (c == '\n'))
/* error  */
      { 
         write_token_lexcode(p, (unsigned int) tk_eof);
         printf("\nerror line %u: unexpected end of string", line_number);
         exit (1000);
      }


/* c != EOF and (c != '\n')  */

                             if (c == '\"')
                             /* end string or "  */
                               {

                                  c = fgetc(lexfile_ptr);
                                  if (c != '\"') 
                                   /* at end of string  */
                                  { 
                                    ungetc(c, lexfile_ptr);
                                    break;
                                  }
                              }
                     
                           /* c to be stored in string  */


                             lexbuf[i] = c;
                             i++;

                             if (i >= BSIZE)
                                /* error  */
                                {
                                   printf("\nerror line %u: string %s... too long", 
line_number,
lexbuf
);
                                    exit (1000);
                                }




                          }  /*  while  */
          


/* found '\"' (string terminator)  */


                   /*  i < BSIZE  */

                     lexbuf[i] = '\0';  

#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 1300: found string: %s",
count,
lexbuf
);
#endif


/* copy string */
 temp = (char *) malloc(sizeof(char) * (i + 1));
 if (temp == NULL) 
 { printf("\nread_new_token: no malloc space for string\n");
   exit (1000);
 }
 /* temp != NULL */
 strcpy(temp, lexbuf);


                    /*  temp != NULL  */
                       
                       p[0] = (void *) tk_string;
                       p[1] = temp;



#if (dbg_flag > 20)
printf("\nread_new_token:%u: token string: %s: p[0] = %u, p[1] = %u, return %u",
count,
temp,
p[0],
p[1],
p[0]
);
#endif

                       return (tk_string);

                 }







          else /* c != EOF */
              /* default: unknown token  */
                  {



                       p[0] = (void *) tk_unk;

              p[1] = (void *) (unsigned int) c;

#if (exit_on_unk > 0)
   printf("\nlexical analysis error in file %s: unknown input symbol '%c' at line %u",
lexan_fname(), 
c,
line_number
);
   exit (1000);
#endif


#if (dbg_flag > 20)
printf("\nread_new_token:%u: ckp 2700: found unk char %c: return %u",
count,
c,
tk_unk
);
#endif


                      return (tk_unk);

                  }




     }   /*  while   */




#if (dbg_flag > 100)
printf("\nread_new_token:%u: ckp 3000: (end)",
count
);
#endif

}  /*   read_new_token    */


